--[[
#### Srikanth Tirumala
#### TeamF1
#### www.TeamF1.com
#### Feb 17, 2011

#### File: autoDetect.lua
#### Description: Detects ISP type.

#### Revisions:
None
modification history:
---------------------
01a,13Jun12,sig Fixed SPR#32543
]]--



--************* Requires *************
require "ispDetectLib"

--package

--table packages

autoDetect = {}

    local ISP_PPPOE           = 1
    local ISP_BIGPOND         = 2
    local ISP_DHCP            = 4
    local ISP_DHCP_DHCPSPOOF  =12
    local ISP_DHCP_PPPOE      = 5
    local ISP_DHCP_PPPOE_PPTP = 21
    local ISP_DHCP_DHCPSPOOF_PPPOE  =13
    local ISP_PPPOE_PPTP      = 17	
    local ISP_DHCPSPOOF       = 8
    local ISP_PPTP            = 16
    local WanStatusMsg 
    local ConnectionType = "dhcpc"
    local RequireLogin = 0
    local dynConf = 0
    local IfStatus = 1

--************* Logic *************

function autoDetect.detectIsp (logicalIfName, addressFamily)
    -- if not allowed to edit
    if (ACCESS_LEVEL ~= 0) then
        return "ACCESS_DENIED", "ADMIN_REQD"
    end

    local LogicalIfName = logicalIfName

    local interfaceName = db.getAttribute ("networkInterface", "LogicalIfName", logicalIfName, "interfaceName")

    local status = ispDetectLib.callLinkIspDetect(interfaceName)

    local where = "LogicalIfName = '" .. logicalIfName .."' and AddressFamily =" .. addressFamily

    local nimfConfMgmtRow = db.getRowWhere ("NimfConf",where) 	

    local f = io.open("/tmp/ispFile")                       -- open file
    local l = f:read("*all")                                -- read file
    local ispFound = tonumber(l)
	f:close()

    if ( ispFound == ISP_BIGPOND ) then
	ConnectionType = "CONN_BIGPOND"
	WanStatusMsg = "BigPond DHCP service detected";
	IfStatus = 1
	RequireLogin = 0

    elseif ((ispFound == ISP_DHCP) or (ispFound == ISP_DHCPSPOOF) or (ispFound == ISP_DHCP_DHCPSPOOF) or (ispFound == ISP_DHCP_DHCPSPOOF_PPPOE) or (ispFound == ISP_DHCP_PPPOE) or (ispFound == ISP_DHCP_PPPOE_PPTP)) then
	ConnectionType = "dhcpc"
	WanStatusMsg = "DHCP service  detected"
	IfStatus = 1
        -- as DHCP service is detected, changing Required Login filed to 'no'
	RequireLogin = 0
        -- changing ISP type to 'DHCP', setting the 'configure' flag 
	dynConf = 1
    local dhcpcRow = db.getRow("Dhcpc","LogicalIfName",logicalIfName)
    if(dhcpcRow ~= nil) then
        dhcpcRow["Dhcpc.GetDnsFromIsp"] = dynConf
        valid = db.update("Dhcpc", dhcpcRow, dhcpcRow["Dhcpc._ROWID_"])
    end

    elseif ((ispFound == ISP_PPPOE) or (ispFound == ISP_DHCP_PPPOE_PPTP)) then
        -- as PPPoE service is detected, changing Required Login filed to 'yes'*/
        ConnectionType = "pppoe"
	WanStatusMsg = "PPPoE service  detected : Please configure necessary fields and click 'Apply' "
	RequireLogin = 1
	IfStatus = 0 
        dynConf = 1
    local pppoeRow = db.getRow("Pppoe","LogicalIfName",logicalIfName)
    if(pppoeRow ~= nil) then
        pppoeRow["Pppoe.GetDnsFromIsp"] = dynConf
        valid = db.update("Pppoe", pppoeRow, pppoeRow["Pppoe._ROWID_"])
    end
        
    elseif (ispFound == ISP_PPTP) then
        -- as PPTP service is detected, changing Required Login filed to 'yes'*/
	ConnectionType = "pptp"
        WanStatusMsg = "PPTP service  detected : Please configure necessary fields and click 'Apply' ";
	RequireLogin = 1        
	IfStatus = 0 
        dynConf = 1
    local pptpRow = db.getRow("Pptp","LogicalIfName",logicalIfName)
    if(pptpRow ~= nil) then
        pptpRow["Pptp.GetDnsFromIsp"] = dynConf
        valid = db.update("Pptp", pptpRow, pptpRow["Pptp._ROWID_"])
    end
        
       --Check if Connection Type is  static
    else	
	if (nimfConfMgmtRow) then
        if ( nimfConfMgmtRow["NimfConf.ConnectionType"] ~= "ifStatic" ) then
	    ConnectionType = "dhcpc"
            WanStatusMsg = "No service detected - Configuration is set to default settings"
	    RequireLogin = 0
	    IfStatus = 1
            dynConf = 1
        else 
	    ConnectionType = "ifStatic"
	    RequireLogin = 0
	    IfStatus = 1
            WanStatusMsg = "No service detected and Static IP information is configured. Please check \
            	            directions from your service Providers"  
        end
	end
    end
	
	if (nimfConfMgmtRow) then
	
	nimfConfMgmtRow["NimfConf.ConnectionType"] = ConnectionType
	nimfConfMgmtRow["NimfConf.RequireLogin"] = RequireLogin
	nimfConfMgmtRow["NimfConf.IfStatus"] = IfStatus

    local wanRowId =	db.existsRowWhere("NimfConf", where)
	valid = db.update("NimfConf", nimfConfMgmtRow, wanRowId)
       if(valid) then 
	  -- commit the transaction
	-- return
	return "OK", WanStatusMsg
	else
	return "ERROR","Operation Failed" 
	end
	end
end --function

