--[[
#### Hussain Vali.
#### TeamF1
#### www.TeamF1.com
#### April 01, 2008
#### File: igmp.lua
#### Description: IGMP Proxy Setup functions
#### Revisions:
01a 23jan13,cg  Fixed SPR #36520 
]]--

--************* Requires *************

--************* Initial Code *************

--package IGMP Proxy Configurations
igmp = {}

--************* Functions *************

-- IGMP Proxy config
function igmp.config (inputTable, rowid, operation)
    -- validate
    local valid = false
    if (ACCESS_LEVEL ~= 0) then
        return "ACCESS_DENIED", "ADMIN_REQD"
    end

    if (db.typeAndRangeValidate(inputTable)) then
        if (operation == "add") then
            return nil
        elseif (operation == "edit") then
            valid = db.update("Igmp", inputTable, rowid)
	    -- return
    	    if (valid) then
        	db.commitTransaction()
        	return "OK", "STATUS_OK"
            else
        	db.rollback()
        	return "ERROR", "IGMP_CONFIG_FAILED"
            end
        elseif (operation == "delete") then
            return nil
        end
    end
    return false
end

-- Delete Allowed Networks
function igmp.deleteAllowedNetwork (rowIds)

	if (ACCESS_LEVEL ~= 0) then
		return "ACCESS_DENIED", "ADMIN_REQD"
	end

	db.beginTransaction() --begin transactions
    local valid = true
    local errStr = "STATUS_OK"

    for k,v in pairs(rowIds) do
        local inTable = {}
        inTable["allowedNets._ROWID_"] = v;
        valid = igmp.networkConfig(inTable, "-1", "delete")
        if (valid == false) then
            errStr = "IGMP_NETWORK_DELETE_FAILED"
            break
        end
    end

    if (valid ~= false) then
        errStr = "STATUS_OK"
    end

	-- return
	if (valid) then
		db.commitTransaction(true)
		return "OK", errStr
	else
		db.rollback()
		return "ERROR", errStr
	end
end

--  Allowed Networks Configuration
function igmp.allowedNetworkConfig (inputTable, rowid, operation)
	-- if not allowed to edit
	if (ACCESS_LEVEL ~= 0) then
		return "ACCESS_DENIED", "ADMIN_REQD"
	end
	db.beginTransaction() --begin transaction
	local valid = false

   	valid = igmp.networkConfig(inputTable, rowid, operation)
	
	-- return
	if (valid) then
		db.commitTransaction(true)
	    return "OK", "STATUS_OK"            
	else
		db.rollback()
		return "ERROR", "ALLOWED_NETWORK_CONFIG_FAILED"
	end
end

-- Igmp Allowed Network Config
function igmp.networkConfig (inputTable, rowid, operation)
    -- validate
    if (db.typeAndRangeValidate(inputTable)) then
        if (operation == "add") then
            return db.insert(DBTable, inputTable)
        elseif (operation == "edit") then
            return db.update(DBTable, inputTable, rowid)
        elseif (operation == "delete") then
            return db.delete(DBTable, inputTable)
        end
    end
    return false
end
