-- l2tpdCLI.lua
-- Ravindranath
-- TeamF1
-- www.TeamF1.com

-- Modification History
-- 26dec12.sig fixed SPR 36175,36176
-- 08may12.nik fixed SPR 31441,31530
-- 16mar12,sig added changes for spr 30595
-- 15mar12,sig fixed for 30595
-- 08dec11,akr added idleTimeout in show l2tp config command
-- 08mar10,rac Added
require "teamf1lualib/l2tpServer"

function l2tpServerCfgInit(args)
	configRow = db.getRow ("Xl2tpServerIPRange", "_ROWID_", "1")
	return 1,configRow
end

function l2tpServerCfgSave(configRow)
	local errorFlag="ERROR"
	local statusCode=""
	local statusMessage=""
	DBTable="Xl2tpServerIPRange"
	errorFlag, statusCode = l2tpServer.serverConfig (configRow, "1", "edit")
	    if (errorFlag == "OK") then db.save() end
		statusMessage = db.getAttribute("stringsMap", "stringId", statusCode, LANGUAGE) or statusCode
	return errorFlag,statusMessage
end

function l2tpServerInputVal(configRow)
    
    if (configRow["Xl2tpServerIPRange.Xl2tpdEnable"] == "1")then
        if (configRow["Xl2tpServerIPRange.StartIPAddress"] == nil or configRow["Xl2tpServerIPRange.StartIPAddress"] == "" or configRow["Xl2tpServerIPRange.StartIPAddress"] == "0.0.0.0")then
            printCLIError("Invalid l2tp server start address range")
            return false
        end
        if (configRow["Xl2tpServerIPRange.EndIPAddress"] == nil or configRow["Xl2tpServerIPRange.EndIPAddress"] == "" or configRow["Xl2tpServerIPRange.EndIPAddress"] == "0.0.0.0")then
            printCLIError("Invalid l2tp server end address range")
            return false
        end
        local startIp = util.split (configRow ["Xl2tpServerIPRange.StartIPAddress"],".")
        local endIp = util.split (configRow ["Xl2tpServerIPRange.EndIPAddress"],".")
        for i = 1,3 do
            if (startIp[i] ~= endIp[i]) then
                printCLIError("Start and End IP Address must be in same subnet.")
                return false
            end
        end
        if (tonumber(startIp[4]) < 1 or tonumber(startIp[4]) > 254) then
            printCLIError("Invalid value for octet 4 in Start IP address. Please enter a value between 1 - 254.")
            return false
        end
        if (tonumber(endIp[4]) < 1 or tonumber(endIp[4]) > 254) then
            printCLIError("Invalid value for octet 4 in End IP address. Please enter a value between 1 - 254.")
            return false
        end
        if (startIp[4] == endIp[4]) then
            printCLIError("End IP Address should not be equal to Start IP Address.")
            return false
        end
        local range = tonumber(endIp[4]) - tonumber(startIp[4])
        if (tonumber(range) > 26) then
            printCLIError("Start and End IP Address range must be less than or equal to 26.")
            return false
        end
        if (tonumber(range) < 0) then
            printCLIError("End IP Address should be higher than Start IP Address.")
            return false
        end
        local pptpStatus = db.getAttribute ("pptpServerIPRange","_ROWID_","1","pptpdEnable")
        if (pptpStatus == "1") then
            local pptpIp = db.getAttribute ("pptpServerIPRange","_ROWID_","1","startIPAddress")
            local status = ifDev.ipSubnet (configRow ["Xl2tpServerIPRange.StartIPAddress"],"255.255.255.0",pptpIp)
            if (status == "OK") then
                printCLIError("PPTP Start IP Address is in the same subnet. Please specify IP address in a different subnet.")
                return false
            end
        end
        if (configRow["Xl2tpServerIPRange.idleTimeOut"] == nil or configRow["Xl2tpServerIPRange.idleTimeOut"] == "")then
            printCLIError("Invalid l2tp server Idle Timeout")
            return false
        end
        if(configRow["Xl2tpServerIPRange.pap"] == "0" and 
            configRow["Xl2tpServerIPRange.chap"]== "0" and 
            configRow["Xl2tpServerIPRange.mschap"]=="0" and 
            configRow["Xl2tpServerIPRange.mschapv2"]=="0") then
            printCLIError("Please select atleast one option from authentication section")
            return false
        end
   
    end
    return true
end

function l2tpdtGet()
	local state
	local resultTab={}
	configRow = db.getRow ("Xl2tpServerIPRange", "_ROWID_", "1")
		printLabel("L2TP Server Configuration")
		if (configRow["Xl2tpServerIPRange.Xl2tpdEnable"] == "0")then
			state="Disabled"
		else
			state="Enabled"
		end 
		resTab.insertField (resultTab,"L2TP Server Status",state)
		resTab.insertField (resultTab,"L2TP Starting IP Address",configRow["Xl2tpServerIPRange.StartIPAddress"] or '')
		resTab.insertField (resultTab,"L2TP server Ending IP Address",configRow["Xl2tpServerIPRange.EndIPAddress"] or '')
		resTab.insertField (resultTab,"L2TP server Idle Timeout",configRow["Xl2tpServerIPRange.idleTimeOut"] or '')
        if (configRow["Xl2tpServerIPRange.Xl2tpdEnable"] == "0") then
            state = "Disabled"
		resTab.insertField (resultTab,"L2TP Server PAP",state)
		resTab.insertField (resultTab,"L2TP Server CHAP",state)
		resTab.insertField (resultTab,"L2TP Server MSCHAP",state)
		resTab.insertField (resultTab,"L2TP Server MSCHAPv2",state)
        else
            if (configRow["Xl2tpServerIPRange.pap"] == "0")then
                state="Disabled"
            else
                state="Enabled"
            end 
            resTab.insertField (resultTab,"PPTP Server PAP",state)
            if (configRow["Xl2tpServerIPRange.chap"] == "0")then
                state="Disabled"
            else
                state="Enabled"
            end 
            resTab.insertField (resultTab,"PPTP Server CHAP",state)
            if (configRow["Xl2tpServerIPRange.mschap"] == "0")then
                state="Disabled"
            else
                state="Enabled"
            end 
            resTab.insertField (resultTab,"PPTP Server MSCHAP",state)
            if (configRow["Xl2tpServerIPRange.mschapv2"] == "0")then
                state="Disabled"
            else
                state="Enabled"
            end 
            resTab.insertField (resultTab,"PPTP Server MSCHAPv2",state)
        end
		resTab.print (resultTab,0)
end

function l2tpdtclientGet()
	local resultTab={}
	local prog = db.getAttribute("environment", "name", "xl2tpdDisplayUserDetails", "value")
	if (prog ~= nil ) then
		os.execute(prog .. " " .. "/tmp/system.db")
	end

	local l2tpSvrEnabled = db.getAttribute ("Xl2tpServerIPRange", "_ROWID_", "1", "Xl2tpdEnable") or '0'
	printLabel("List of L2TP Active Users")
	if (l2tpSvrEnabled == "1") then
		local i = 0
		local table = db.getTable("Xl2tpActiveUsers")
		for k,v in pairs(table) do
			i = i + 1							
			local row = table[i]							
				resTab.insertField (resultTab,"User Name",row["Xl2tpActiveUsers.userName"] or '')
				resTab.insertField (resultTab,"Remote IP",row["Xl2tpActiveUsers.RemoteIP"] or '')
				resTab.insertField (resultTab,"L2TP IP",row["Xl2tpActiveUsers.Xl2tpIP"] or '')
		end
		resTab.print (resultTab,0)
	end
end
