-- pptpdCLI.lua
-- Ravindranath
-- TeamF1
-- www.TeamF1.com

-- Modification History
-- 01a, 26dec12, fixed Spr 36176 
-- 08mar10,rac Added
require "teamf1lualib/pptpServer"


function pptpServerCfgInit(args)
	configRow = db.getRow ("pptpServerIPRange", "_ROWID_", "1")
	return 1,configRow
end

function pptpServerCfgSave(configRow)
	local errorFlag="ERROR"
	local statusCode=""
	local statusMessage=""
	DBTable="pptpServerIPRange"
	errorFlag, statusCode = pptpServer.serverConfig (configRow, "1", "edit")
	    if (errorFlag == "OK") then db.save() end
		statusMessage = db.getAttribute("stringsMap", "stringId", statusCode, LANGUAGE) or statusCode
	return errorFlag,statusMessage
end

function pptpServerInputVal(configRow)
    
    if (configRow["pptpServerIPRange.pptpdEnable"] == "1")then
        if (configRow["pptpServerIPRange.startIPAddress"] == nil or configRow["pptpServerIPRange.startIPAddress"] == "" or configRow["pptpServerIPRange.startIPAddress"] == "0.0.0.0")then
            printCLIError("Invalid pptp server start address range")
            return false
        end
        if (configRow["pptpServerIPRange.endIPAddress"] == nil or configRow["pptpServerIPRange.endIPAddress"] == "" or configRow["pptpServerIPRange.endIPAddress"] == "0.0.0.0")then
            printCLIError("Invalid pptp server  end address range")
            return false
        end
        local startIp = util.split (configRow ["pptpServerIPRange.startIPAddress"],".")
        local endIp = util.split (configRow ["pptpServerIPRange.endIPAddress"],".")
        for i = 1,3 do
            if (startIp[i] ~= endIp[i]) then
                printCLIError("Start and End IP Address must be in same subnet.")
                return false
            end
        end
        if (tonumber(startIp[4]) < 1 or tonumber(startIp[4]) > 254) then
            printCLIError("Invalid value for octet 4 in Start IP address. Please enter a value between 1 - 254.")
            return false
        end
        if (tonumber(endIp[4]) < 1 or tonumber(endIp[4]) > 254) then
            printCLIError("Invalid value for octet 4 in End IP address. Please enter a value between 1 - 254.")
            return false
        end
        if (startIp[4] == endIp[4]) then
            printCLIError("End IP Address should not be equal to Start IP Address.")
            return false
        end
        local range = tonumber(endIp[4]) - tonumber(startIp[4])
        if (tonumber(range) > 26) then
            printCLIError("Start and End IP Address range must be less than or equal to 26.")
            return false
        end
        if (tonumber(range) < 0) then
            printCLIError("End IP Address should be higher than Start IP Address.")
            return false
        end
        local l2tpStatus = db.getAttribute ("Xl2tpServerIPRange","_ROWID_","1","Xl2tpdEnable")
        if (l2tpStatus == "1") then
            local l2tpIp = db.getAttribute ("Xl2tpServerIPRange","_ROWID_","1","StartIPAddress")
            local status = ifDev.ipSubnet (configRow ["pptpServerIPRange.startIPAddress"],"255.255.255.0",l2tpIp)
            if (status == "OK") then
                printCLIError("L2TP Start IP Address is in the same subnet. Please specify IP address in a different subnet.")
                return false
            end
        end
        if (configRow["pptpServerIPRange.idleTimeOut"] == nil or configRow["pptpServerIPRange.idleTimeOut"] == "")then
            printCLIError("Invalid pptp server Idle Timeout")
            return false
        end
        if(configRow["pptpServerIPRange.pap"]== "0" and configRow["pptpServerIPRange.chap"]== "0" and configRow["pptpServerIPRange.mschap"]=="0" and configRow["pptpServerIPRange.mschapv2"]=="0") then
            printCLIError("Please select atleast one option from authentication section")
            return false
        end
        if(configRow["pptpServerIPRange.mschap"]=="1" and configRow["pptpServerIPRange.mppe40"] == "1") then
            printCLIError("MPPE 40-bit encryption not supported for MSCHAP authentication")
            return false
        end
   
    end
    return true
end
function pptpdtGet()
	local state
	local resultTab={}
	configRow = db.getRow ("pptpServerIPRange", "_ROWID_", "1")
		printLabel("PPTP Server Configuration")
		if (configRow["pptpServerIPRange.pptpdEnable"] == "0")then
			state="Disabled"
		else
			state="Enabled"
		end 
		resTab.insertField (resultTab,"PPTP Server Status",state)
		resTab.insertField (resultTab,"PPTP Starting IP Address",configRow["pptpServerIPRange.startIPAddress"] or '')
		resTab.insertField (resultTab,"PPTP server Ending IP Address",configRow["pptpServerIPRange.endIPAddress"] or '')
		resTab.insertField (resultTab,"PPTP server Idle Timeout",configRow["pptpServerIPRange.idleTimeOut"] or '')
        if( configRow["pptpServerIPRange.pptpdEnable"] == "0") then
            state = "Disabled"
            resTab.insertField (resultTab,"PPTP Server PAP",state)
            resTab.insertField (resultTab,"PPTP Server CHAP",state)
            resTab.insertField (resultTab,"PPTP Server MSCHAP",state)
            resTab.insertField (resultTab,"PPTP Server MSCHAPv2",state)
        else
            if (configRow["pptpServerIPRange.pap"] == "0")then
                state="Disabled"
            else
                state="Enabled"
            end 
            resTab.insertField (resultTab,"PPTP Server PAP",state)
            if (configRow["pptpServerIPRange.chap"] == "0")then
                state="Disabled"
            else
                state="Enabled"
            end 
            resTab.insertField (resultTab,"PPTP Server CHAP",state)
            if (configRow["pptpServerIPRange.mschap"] == "0")then
                state="Disabled"
            else
                state="Enabled"
            end 
            resTab.insertField (resultTab,"PPTP Server MSCHAP",state)
            if (configRow["pptpServerIPRange.mschapv2"] == "0")then
                state="Disabled"
            else
                state="Enabled"
            end 
            resTab.insertField (resultTab,"PPTP Server MSCHAPv2",state)
        end
        if(configRow["pptpServerIPRange.mschap"] == "0" and configRow["pptpServerIPRange.mschapv2"] == "0") then
            state ="Disabled"
            resTab.insertField (resultTab,"PPTP Server MPPE40",state)
            resTab.insertField (resultTab,"PPTP Server MPPE128",state)
            resTab.insertField (resultTab,"PPTP Server MPPE Stateful",state)
        else
            if (configRow["pptpServerIPRange.mppe40"] == "0")then
                state="Disabled"
            else
                state="Enabled"
            end 
            resTab.insertField (resultTab,"PPTP Server MPPE40",state)
            if (configRow["pptpServerIPRange.mppe128"] == "0")then
                state="Disabled"
            else
                state="Enabled"
            end 
            resTab.insertField (resultTab,"PPTP Server MPPE128",state)
            if (configRow["pptpServerIPRange.mppeStateful"] == "0")then
                state="Disabled"
            else
                state="Enabled"
            end 
            resTab.insertField (resultTab,"PPTP Server MPPE Stateful",state)
        end
		resTab.print (resultTab,0)
end

function pptpdtclientGet()
	local resultTab={}
    pptpServer.display ()
	local pptpSvrEnabled = db.getAttribute ("pptpServerIPRange", "_ROWID_", "1", "pptpdEnable") or '0'
	printLabel("List of PPTP Active Users")
	if (pptpSvrEnabled == "1") then
		local i = 0
		local table = db.getTable("pptpServerUsers")
		for k,v in pairs(table) do
			i = i + 1							
			local row = table[i]							
				resTab.insertField (resultTab,"User Name",row["pptpServerUsers.userName"] or '')
				resTab.insertField (resultTab,"Remote IP",row["pptpServerUsers.remoteIP"] or '')
				resTab.insertField (resultTab,"PPTP IP",row["pptpServerUsers.pptpIP"] or '')
		end
		resTab.print (resultTab,0)
	end
end
