--[[
#### Abhinendra Babu. T
#### TeamF1
#### www.TeamF1.com
#### May 23, 2008
#### File: sslAdmin.lua
#### Description: SSL functions (domains, groups and users)
#### Revisions:
10c, 12dec12, pbr, fixed spr 36016
01b, 21nov08, til, included new functions as a fix for spr-9484
]]--

--************* Requires *************
require "teamf1lualib/sslvpn"

--************* Initial Code *************

--package Admin Configurations
sslAdmin = {}

--************* Functions *************


-- cleanUp routines:
--
--Clean up user's Login Policies Related Tables(DefinedAddress, DefinedBrowser)
--
--
function sslAdmin.deleteLoginPolicyTablesOfUser (userName)
    local valid = true
    
    -- Clean up DefinedAddress Table records for this user
    local table = db.getRows ("SSLVPNUserLoginPolicyDefinedAddress", 
                              "UserName", userName)

    -- if no DefinedAddress  are configured for this user return as success
    if (table == nil) then
        return true
    end
    
	local i = 0
    for key,value in pairs(table) do
	    i = i + 1
	    local row = table[i]
        local delTable = {}
        local rowUserName = row["SSLVPNUserLoginPolicyDefinedAddress.UserName"]
        local rowId = db.existsRow ("SSLVPNUserLoginPolicyDefinedAddress", 
                                    "UserName", rowUserName)
        if (rowId) then
            delTable["SSLVPNUserLoginPolicyDefinedAddress._ROWID_"] = rowId 
            valid = db.delete ("SSLVPNUserLoginPolicyDefinedAddress", delTable)
	        if (not valid) then
                break
		    end
        end    
    end-- for each DefinedAddressTbl record

    -- if  deletion of DefinedAddressTbl failed, return as Failed
    if (not valid) then
        return (valid)
    end    
    
    --
    --Clean up DefinedBrowser Tble records for this user
    --
    local table = db.getRows ("SSLVPNUserLoginPolicyDefinedBrowser", 
                              "UserName", userName)
    -- if Policies are configured for this owner return as success
    if (table == nil) then
        return true
    end
    
    -- now that table is not nil, ie., we have policies configured for this
    -- owner
	local i = 0
    for key,value in pairs(table) do
	    i = i + 1
	    local row = table[i]
        local delTable = {}
        local rowUserName = row["SSLVPNUserLoginPolicyDefinedBrowser.UserName"]
        local rowId = db.existsRow ("SSLVPNUserLoginPolicyDefinedBrowser", 
                                    "UserName", rowUserName)
        if (rowId) then
            delTable["SSLVPNUserLoginPolicyDefinedBrowser._ROWID_"] = rowId 
            valid = db.delete ("SSLVPNUserLoginPolicyDefinedBrowser", delTable)
	        if (not valid) then
                break
		    end
        end    
    end-- for each DefinedBrowserTbl record

   return (valid)
end

--
--Clean Up associated Verisign Credentials, hence resolve the dependencies.
--
--Either delete the cred-record or dis-associate the user-to-credID
--
--Clean-up the userName for the respective cred-record:
--
function sslAdmin.disAssociateUserFromVsignCredential (userName)
    local valid = true
    local vsignRowWithUser = db.existsRow ("vsign2fctrCredMgmt", "userName", 
                                           userName)
    -- if NO credetials are configured for this user return as success
    if (vsignRowWithUser == nil) then
        return true
    end
    
    -- now that vsignRowWithUser is not nil, ie., we have policies configured 
    -- for this owner
    if (vsignRowWithUser) then
        valid = db.setAttribute ("vsign2fctrCredMgmt", "_ROWID_", 
                                 vsignRowWithUser, "userName", "")
    end
    return (valid)  
end

--
--Clean Up associated sslvpn Policies, hence resolve the dependencies
--between deleted policies and resources. Like update reference count 
--of policies and resources etc..,
--
--see : policies/resources specs
--
--Clean up sslvpn-policy records for this owner
--
function sslAdmin.deleteSSLVPNPoliciesOfOwner (ownerType, ownerName)
    local valid = true  
    local policyWherePart = "PolicyType = '" .. ownerType .. "' AND PolicyOwnerName = '" .. ownerName .. "'"
    local table = db.getRowsWhere ("SSLVPNPolicy", policyWherePart)
    -- if Policies are configured for this owner return as success
    if (table == nil) then
        return true
    end
    
    -- now that table is not nil, ie., we have policies configured for this
    -- owner
    local i = 0
    for key,value in pairs(table) do
        i = i + 1
	    local row = table[i]
        local thisRowPolicyName = row["SSLVPNPolicy.PolicyName"]
        local thisRowWherePart = "PolicyType = '" .. ownerType .. "' AND PolicyOwnerName = '" .. ownerName .. "' AND PolicyName = '" .. thisRowPolicyName .. "'"
        
        local rowId = db.existsRowWhere ("SSLVPNPolicy", thisRowWherePart)
        local delTable = {}
        if (rowId) then
            delTable["SSLVPNPolicy._ROWID_"] = rowId
            -- get the destTYpe of this policy
            local destType = db.getAttribute ("SSLVPNPolicy", "_ROWID_", rowId,
                                              "DestinationObjectType")
            -- if the destTYpe is 'networkResource' get its Name
            -- and hence reduce its referenceCount as we are about to 
            -- delete the associated policy
            if (destType == "1") then
                local resourceName = db.getAttribute ("SSLVPNPolicy",
                                                      "_ROWID_", rowId,
                                                      "ResourceName")
                valid = sslvpn.updateResource(resourceName, "delete");
            else
                valid = true
            end
            
            if (valid) then
	            valid = db.delete("SSLVPNPolicy", delTable)
            end
        end --if uniq-rowId
        
        if (not valid) then
            break
		end
    end-- for each policy record

    return (valid)
end


-- Domain config
function sslAdmin.domainConfig (inputTable, rowid, operation)
    -- if not allowed to edit
    if (ACCESS_LEVEL ~= 0) then
    	return "ACCESS_DENIED", "ADMIN_REQD"
    end
    db.beginTransaction() --begin transaction
    local valid = false

    -- Insertion / Updation here
    --
    if (db.existsRow ("SSLVPNPortalLayout", "PortalLayoutName", inputTable["USERDBDomains.PortalLayoutName"])) then
	    local portalRowId = db.getAttribute ("SSLVPNPortalLayout", "PortalLayoutName", inputTable["USERDBDomains.PortalLayoutName"], "_ROWID_")
    	local portalTable = {}
	    if (operation == "add") then
             if(inputTable["USERDBDomains.AuthenticationType"] == "active-directory" and inputTable["USERDBDomains.ActiveDirectoryDomain"] ~= nil) then
   if(db.existsRow("USERDBDomains", "ActiveDirectoryDomain", inputTable["USERDBDomains.ActiveDirectoryDomain"])) then
    db.rollback()
    return "ERROR", "SSLVPN_DUPLICATE_DOMAIN_CONFIG_FAILED"
    end
    end
                           	local useCount =  db.getAttribute ("SSLVPNPortalLayout", "_ROWID_", portalRowId, "UseCount")
	       	 valid = db.setAttribute("SSLVPNPortalLayout", "_ROWID_", portalRowId, "UseCount", tonumber(useCount) + 1)
        elseif (operation == "edit") then
            if(inputTable["USERDBDomains.AuthenticationType"] == "active-directory" and inputTable["USERDBDomains.ActiveDirectoryDomain"] ~= nil) then
        local duplicateDomain = "ActiveDirectoryDomain = '" .. inputTable["USERDBDomains.ActiveDirectoryDomain"] .. "'  AND _ROWID_ !=  '" .. rowid .. "'"

             if(db.existsRowWhere("USERDBDomains", duplicateDomain)) then
                 db.rollback()
                 return "ERROR", "SSLVPN_DUPLICATE_DOMAIN_CONFIG_FAILED"
            end
           end
                        
            	local oldPortal = db.getAttribute ("USERDBDomains", "_ROWID_", rowid, "PortalLayoutName")
            	local newPortal = inputTable["USERDBDomains.PortalLayoutName"]
            	if (oldPortal ~= newPortal) then
              	  -- Decrement Previous portal usecount
        	  	  local PortalName = db.getAttribute ("USERDBDomains", "_ROWID_", rowid, "PortalLayoutName")
	           	   portalRowId = db.getAttribute ("SSLVPNPortalLayout", "PortalLayoutName", PortalName, "_ROWID_")
               		 local useCount =  db.getAttribute ("SSLVPNPortalLayout", "_ROWID_", portalRowId, "UseCount")
                	if (tonumber(useCount) - 1 == 0) then
	                	local isDefault = db.getAttribute("SSLVPNPortalLayout", "_ROWID_", portalRowId, "DefaultPortal")
                   		 if (isDefault == "2") then
                       		 db.rollback()
        	            	return "ERROR", "SSLVPN_PORTAL_IS_DEFAULT_DOMAIN_EDIT_FAILED"
                    	end
                	end
	           		 valid = db.setAttribute("SSLVPNPortalLayout", "_ROWID_", portalRowId, "UseCount", tonumber(useCount) - 1)
    	       		 if (valid) then
	        	   		 -- Increment portal usecount
    	    	   		 portalRowId = db.getAttribute ("SSLVPNPortalLayout", "PortalLayoutName", inputTable["USERDBDomains.PortalLayoutName"], "_ROWID_")
                   		 useCount =  db.getAttribute ("SSLVPNPortalLayout", "_ROWID_", portalRowId, "UseCount")
	               		 valid = db.setAttribute("SSLVPNPortalLayout", "_ROWID_", portalRowId, "UseCount", tonumber(useCount) + 1)
	            	end
            	else
                	valid = true
            	end
       elseif (operation == "delete") then
            valid = false
       end
    end

    if (valid) then
        -- validate
        if (db.typeAndRangeValidate(inputTable)) then
            if (operation == "add") then
                valid = db.insert(DBTable, inputTable)
            elseif (operation == "edit") then
                valid = db.update(DBTable, inputTable, rowid)
            elseif (operation == "delete") then
                valid = false
            end
        end
    end

    local inTable = {}
    if (valid) then
	inTable["USERDBGroups.GroupName"] = inputTable["USERDBDomains.DomainName"]
	inTable["USERDBGroups.DomainName"] = inputTable["USERDBDomains.DomainName"]
	inTable["USERDBGroups.GroupTimeOut"] = inputTable["USERDBDomains.TimeOut"]
	inTable["USERDBGroups.DefaultGroup"] = 1
	if (operation == "add") then
            valid = db.insert("USERDBGroups", inTable)
        end
    end

    -- return
    if (valid) then
        db.commitTransaction(true)
        return "OK", "STATUS_OK"
    else
        db.rollback()
        return "ERROR", "SSLVPN_DOMAIN_CONFIG_FAILED"
    end
end

-- Domains Deletion
function sslAdmin.deleteDomains (rowIds)
    -- if not allowed to edit
    if (ACCESS_LEVEL ~= 0) then
    	return "ACCESS_DENIED", "ADMIN_REQD"
    end
    
    db.beginTransaction() --begin transaction
    local valid = false

    -- delete rows
    for k,v in pairs(rowIds) do    	
   	    local inTable = {}
    	local portalTable = {}
	    inTable["USERDBDomains._ROWID_"] = v;
	
    	-- get portal name and portal row id
	    local PortalName = db.getAttribute ("USERDBDomains", "_ROWID_", v, "PortalLayoutName")
        local portalRowId = db.getAttribute ("SSLVPNPortalLayout", "PortalLayoutName", PortalName, "_ROWID_")
	
    	-- check if its default portal and it is the only domain.
        local defaultPortalName = db.getAttribute ("SSLVPNPortalLayout","DefaultPortal","2","PortalLayoutName")
        if (defaultPortalName == nil) then
            defaultPortalName = db.getAttribute ("SSLVPNPortalLayout","DefaultPortal","1","PortalLayoutName")
        end
        if (defaultPortalName == PortalName) then
            if (tonumber(db.getAttribute ("SSLVPNPortalLayout", "_ROWID_", portalRowId, "UseCount")) == 1) then
                db.rollback()
    	        return "ERROR", "SSLVPN_PORTAL_IS_DEFAULT_DOMAIN_DEL_FAILED"
            end
    	end
	
    	-- Decrement Use count
        local useCount =  db.getAttribute ("SSLVPNPortalLayout", "_ROWID_", portalRowId, "UseCount")
        valid = db.setAttribute("SSLVPNPortalLayout", "_ROWID_", portalRowId, "UseCount", tonumber(useCount) - 1)
	
    	-- delete row
	    local domName = db.getAttribute ("USERDBDomains", "_ROWID_", v, "DomainName")
    	if (valid) then
	        local table = db.getRows ("USERDBGroups", "DomainName", domName)
	        local i = 0
    	    for key,value in pairs(table) do
	            i = i + 1
		        local row = table[i]
        		if (db.existsRow ("USERDBUsers", "GroupName", row["USERDBGroups.GroupName"])) then
		            valid = false
        	    else
		            local delTable = {}
        		    delTable["USERDBGroups._ROWID_"] = db.existsRow ("USERDBGroups", "GroupName", row["USERDBGroups.GroupName"])
		            valid = db.delete("USERDBGroups", delTable)
                    -- delete sslvpnPolicies for this group, if any
                    if (valid) then
                    local rowGroupName = row["USERDBGroups.GroupName"]
                    valid = sslAdmin.deleteSSLVPNPoliciesOfOwner ("group",
                                                                  rowGroupName)
                    end
                    
        		end
		        if (not valid) then
        		    break
		        end
    	    end
	        if (valid) then
	            valid = db.delete("USERDBDomains", inTable)
    	    end
	    end

        if (not valid) then
	        break
    	end
    end

    -- return
    if (valid) then
        db.commitTransaction(true)
        return "OK", "STATUS_OK"
    else
        db.rollback()
        return "ERROR", "SSLVPN_DOMAIN_HAS_USERS_DOM_DEL_FAILED"
    end
end

-- Groups config
function sslAdmin.groupConfig (inputTable, rowid, operation)
    -- if not allowed to edit
    if (ACCESS_LEVEL ~= 0) then
    	return "ACCESS_DENIED", "ADMIN_REQD"
    end
    db.beginTransaction() --begin transaction
    local valid = false
    
    -- Insertion / Updation here
    -- validate
    if (db.typeAndRangeValidate(inputTable)) then
        if (operation == "add") then
            valid = db.insert(DBTable, inputTable)
        elseif (operation == "edit") then
            valid = db.update(DBTable, inputTable, rowid)
        elseif (operation == "delete") then
            valid = false
        end
    end

    -- return
    if (valid) then
        db.commitTransaction(true)
        return "OK", "STATUS_OK"
    else
        db.rollback()
        return "ERROR", "SSLVPN_GROUP_CONFIG_FAILED"
    end
end

-- Groups Deletion
function sslAdmin.deleteGroups (rowIds)
    -- if not allowed to edit
    if (ACCESS_LEVEL ~= 0) then
	return "ACCESS_DENIED", "ADMIN_REQD"
    end
    db.beginTransaction() --begin transaction
    local valid = false

    -- delete rows
    for k,v in pairs(rowIds) do
	local inTable = {}
	inTable["USERDBGroups._ROWID_"] = v;

	-- Don't delete if its default group
	local defaultValue = db.getAttribute("USERDBGroups", "_ROWID_", v, "DefaultGroup")
	if (defaultValue == '1') then
	    return "ERROR", "SSLVPN_GROUP_IS_DEFAULT_DEL_FAILED"
	end

	-- Don't delete if it has users
	local groupName = db.getAttribute("USERDBGroups", "_ROWID_", v, "GroupName")
	if (db.existsRow ("USERDBUsers", "GroupName", groupName)) then
	    return "ERROR", "SSLVPN_GROUP_HAS_USER_DEL_FAILED"
	end

    --delete associated sslvpn Policies, 
    valid = sslAdmin.deleteSSLVPNPoliciesOfOwner ("group", groupName)
    if (not valid) then
        break
    end
    
	-- Delete Row
	valid = db.delete("USERDBGroups", inTable)
	if (not valid) then
	    break
	end
    end

    -- return
    if (valid) then
        db.commitTransaction(true)
        return "OK", "STATUS_OK"
    else
        db.rollback()
        return "ERROR", "SSLVPN_GROUP_HAS_USER_DEL_FAILED"
    end
end

-- Users config
function sslAdmin.userConfig (inputTable, rowid, operation)
    -- if not allowed to edit
    if (ACCESS_LEVEL ~= 0) then
    	return "ACCESS_DENIED", "ADMIN_REQD"
    end
    db.beginTransaction() --begin transaction
    local valid = false
   	local oldUname = ""
    local newUname = inputTable["USERDBUsers.UserName"]
    
    -- Changing user type to 1 : external user
    -- if user type selected as sslvpn and authentication
    -- type is not local.
    local domainName = ''
    if (inputTable["USERDBUsers.GroupName"] ~= nil and inputTable["USERDBUsers.GroupName"] ~= "") then
        domainName = db.getAttribute("USERDBGroups","GroupName",inputTable["USERDBUsers.GroupName"],"DomainName") or ''
    end
    if (operation == "add" and domainName ~= '' and domainName ~= nil 
        and (db.getAttribute("USERDBDomains","DomainName",domainName,"AuthenticationType") ~= "local")) then
       --[[ if (inputTable["USERDBUsers.UserType"] == "0") then
            local domainName = db.getAttribute ("USERDBGroups","USERDBGroups.GroupName",inputTable["USERDBUsers.GroupName"],"DomainName")
            if (domainName ~= nil) then
                local authType = db.getAttribute ("USERDBDomains","USERDBDomains.DomainName",domainName,"AuthenticationType")
                if (authType ~= nil and authType ~= "local") then
                    inputTable["USERDBUsers.UserType"] = "1"
                end
            end
        end -- ]] 
        local domainTbl = db.getRowsWhere ("USERDBDomains","AuthenticationType!='local'")
        local i = 0
        local count = 0
        for k,v in pairs(domainTbl) do
            i = i + 1
            local row = domainTbl[i]
            if (row ~= nil) then
                local groups = db.getRows ("USERDBGroups","DomainName",row["USERDBDomains.DomainName"])
                local j = 0
                for k,v in pairs(groups) do
                    j = j + 1
                    local row1 = groups[j]
                    if (row1 ~= nil) then
                        local users = db.getRows ("USERDBUsers","GroupName",row1["USERDBGroups.GroupName"])
                        count = count + (#users)
                    end
                end
            end
        end
        if (count >= 128) then
            db.rollback()
            return "ERROR", "EXT_USER_COUNT_EXCEEDS_MAX_LIMIT"
        end
	if (inputTable["USERDBUsers.UserType"] == "3" or inputTable["USERDBUsers.UserType"] == "4") then
		-- inputTable["USERDBUsers.DenyLogin"] = 1
		inputTable["USERDBUsers.DenyLoginFromWan"] = 1
	end
       		
    elseif (operation == "edit") then
        local oldUsertype = db.getAttribute ("USERDBUsers", "rowid", rowid, "UserType")

	if (tonumber(oldUsertype) ~= tonumber(inputTable["USERDBUsers.UserType"])) then
        if (inputTable["USERDBUsers.UserType"] == "3" or inputTable["USERDBUsers.UserType"] == "4") then
		    -- inputTable["USERDBUsers.DenyLogin"] = 1
		    inputTable["USERDBUsers.DenyLoginFromWan"] = 1
        elseif (inputTable["USERDBUsers.UserType"] == "0")then
            inputTable["USERDBUsers.DenyLoginFromWan"] = 0
	    end
	end

        oldUname = db.getAttribute ("USERDBUsers", "_ROWID_", rowid, "UserName")
        --Checking if it is runtime user
        --Updating CaptivePortalSession table accordingly
        if (oldUsertype == "10") then
            local rtrowid = db.getAttribute("CaptivePortalSession", "UserName", oldUname, "_ROWID_")
            if (rtrowid ~= nil) then  
                if (inputTable["USERDBUsers.UserType"] == "10") then
                    local userRow = db.getRow('CaptivePortalSession', "_ROWID_", rtrowid)
                    userRow["CaptivePortalSession.idleTimeout"] = inputTable["USERDBUsers.UserTimeOut"] 
                    db.update ("CaptivePortalSession", userRow, rtrowid)
                else
                    local temptable = {}
                    temptable["CaptivePortalSession._ROWID_"] = rtrowid
                    db.delete ("CaptivePortalSession",temptable)
                end
            end
        end   

    	--Checking the users own password to change any users password.
        if (inputTable["USERDBUsers.PasswordEdit"] == "1" and inputTable["USERDBUsers.MyPassword"] ~= nil) then
            local wherePart = "LoginAddress = '" .. SAPI.Request.servervariable("REMOTE_ADDR") .. "' AND Cookie = '" .. cgilua.cookies.get("TeamF1Login") .. "'"
            local row = db.getRowWhere("SSLVPNActiveUsers", wherePart)
            if (row ~= nil) then
                local userPwd = db.getAttribute ("USERDBUsers", "UserName", row["SSLVPNActiveUsers.UserName"], "Password")
                if (userPwd ~= inputTable["USERDBUsers.MyPassword"]) then
                    db.rollback()
                    return "ERROR", "WRONG_USER_PASSWORD"
                end
            else
                db.rollback()
                return "ERROR", "USER_SESSION_NOT_FOUND"
            end
        end
    end

    -- Insertion / Updation here
    -- validate
    if (db.typeAndRangeValidate(inputTable)) then
        if (operation == "add") then
            valid = db.insert(DBTable, inputTable)
        elseif (operation == "edit") then
            valid = db.update(DBTable, inputTable, rowid)
        elseif (operation == "delete") then
            valid = false
        end
    end

    if (valid and operation == "edit" and oldUname ~= newUname) then
        if (db.existsRow ("SSLVPNUserLoginPolicyDefinedBrowser", "UserName", oldUname)) then
    	    valid = valid and db.setAttribute("SSLVPNUserLoginPolicyDefinedBrowser", "UserName", oldUname, "UserName", newUname)
        end
        if (db.existsRow ("SSLVPNUserLoginPolicyDefinedAddress", "UserName", oldUname)) then
    	    valid = valid and db.setAttribute("SSLVPNUserLoginPolicyDefinedAddress", "UserName", oldUname, "UserName", newUname)
        end
        if (db.existsRow ("SSLVPNPolicy", "PolicyOwnerName", oldUname)) then
    	    valid = valid and db.setAttribute("SSLVPNPolicy", "PolicyOwnerName", oldUname, "PolicyOwnerName", newUname)
        end
        if (db.existsRow ("vsign2fctrCredMgmt", "UserName", oldUname)) then
    	    valid = valid and db.setAttribute("vsign2fctrCredMgmt", "UserName", oldUname, "UserName", newUname)
        end
    end

    -- return
    if (valid) then
        db.commitTransaction(true)
        return "OK", "STATUS_OK"
    else
        db.rollback()
        return "ERROR", "SSLVPN_USER_CONFIG_FAILED"
    end
end

--XML user Configuration
function sslAdmin.userConfigXML (inputTable, rowid, operation)
    -- if not allowed to edit
    if (ACCESS_LEVEL ~= 0) then
    	return "ACCESS_DENIED", "ADMIN_REQD"
    end
    db.beginTransaction() --begin transaction
    local valid = false
   	local oldUname = ""
    local newUname = inputTable["USERDBUsers.UserName"]
    
    -- Changing user type to 1 : external user
    -- if user type selected as sslvpn and authentication
    -- type is not local.
    if (operation == "add") then
        if (inputTable["USERDBUsers.UserType"] == "0") then
            local domainName = db.getAttribute ("USERDBGroups","USERDBGroups.GroupName",inputTable["USERDBUsers.GroupName"],"DomainName")
            if (domainName ~= nil) then
                local authType = db.getAttribute ("USERDBDomains","USERDBDomains.DomainName",domainName,"AuthenticationType")
                if (authType ~= nil and authType ~= "local") then
                    inputTable["USERDBUsers.UserType"] = "1"
                end
            end
        end
	if (inputTable["USERDBUsers.UserType"] == "4") then
		inputTable["USERDBUsers.DenyLogin"] = 1
		inputTable["USERDBUsers.DenyLoginFromWan"] = 1
	end			
    elseif (operation == "edit") then
        oldUname = db.getAttribute ("USERDBUsers", "_ROWID_", rowid, "UserName")
    	--Checking the users own password to change any users password.
        if (inputTable["USERDBUsers.PasswordEdit"] == "1" and inputTable["USERDBUsers.MyPassword"] ~= nil) then
            local wherePart = "LoginAddress = '" .. SAPI.Request.servervariable("REMOTE_ADDR") .. "' AND Cookie = '" .. cgi.sessionId .. "'"
            local row = db.getRowWhere("SSLVPNActiveUsers", wherePart)
            if (row ~= nil) then
                local userPwd = db.getAttribute ("USERDBUsers", "UserName", row["SSLVPNActiveUsers.UserName"], "Password")
                if (userPwd ~= inputTable["USERDBUsers.MyPassword"]) then
                    db.rollback()
                    return "ERROR", "WRONG_USER_PASSWORD"
                end
            else
                db.rollback()
                return "ERROR", "USER_SESSION_NOT_FOUND"
            end
        end
    end

    -- Insertion / Updation here
    -- validate
    if (db.typeAndRangeValidate(inputTable)) then
        if (operation == "add") then
            valid = db.insert(DBTable, inputTable)
        elseif (operation == "edit") then
            valid = db.update(DBTable, inputTable, rowid)
        elseif (operation == "delete") then
            valid = false
        end
    end

    if (valid and operation == "edit" and oldUname ~= newUname) then
        if (db.existsRow ("SSLVPNUserLoginPolicyDefinedBrowser", "UserName", oldUname)) then
    	    valid = valid and db.setAttribute("SSLVPNUserLoginPolicyDefinedBrowser", "UserName", oldUname, "UserName", newUname)
        end
        if (db.existsRow ("SSLVPNUserLoginPolicyDefinedAddress", "UserName", oldUname)) then
    	    valid = valid and db.setAttribute("SSLVPNUserLoginPolicyDefinedAddress", "UserName", oldUname, "UserName", newUname)
        end
        if (db.existsRow ("SSLVPNPolicy", "PolicyOwnerName", oldUname)) then
    	    valid = valid and db.setAttribute("SSLVPNPolicy", "PolicyOwnerName", oldUname, "PolicyOwnerName", newUname)
        end
        if (db.existsRow ("vsign2fctrCredMgmt", "UserName", oldUname)) then
    	    valid = valid and db.setAttribute("vsign2fctrCredMgmt", "UserName", oldUname, "UserName", newUname)
        end
    end

    -- return
    if (valid) then
        db.commitTransaction(true)
        return "OK", "STATUS_OK"
    else
        db.rollback()
        return "ERROR", "SSLVPN_USER_CONFIG_FAILED"
    end
end

-- Users Deletion
function sslAdmin.deleteUsers (rowIds)
    -- if not allowed to edit
    if (ACCESS_LEVEL ~= 0) then
	return "ACCESS_DENIED", "ADMIN_REQD"
    end
    db.beginTransaction() --begin transaction
    local valid = false

    -- delete rows
    for k,v in pairs(rowIds) do
	local inTable = {}
	inTable["USERDBUsers._ROWID_"] = v;
	local userName = db.getAttribute("USERDBUsers", "_ROWID_", v, "UserName")

    --
    --Resolve Dependencies :
    --
    valid = sslAdmin.disAssociateUserFromVsignCredential (userName)
    if (not valid) then
        break
    end
    
    valid = sslAdmin.deleteLoginPolicyTablesOfUser (userName)
    if (not valid) then
        break
    end
    
    valid = sslAdmin.deleteSSLVPNPoliciesOfOwner ("user", userName)
    if (not valid) then
        break
    end

    -- if the row is of runtimeauthentication then delete in
    -- captiveportalSession also
    local userType = db.getAttribute ("USERDBUsers", "_ROWID_", v, "UserType")
    if (userType == "10") then
        local rtrowid  = db.getAttribute("CaptivePortalSession", "UserName", userName,"_ROWID_")            
		if (rtrowid ~= nil) then
            local temptable = {}
            temptable["CaptivePortalSession._ROWID_"] = rtrowid
            db.delete ("CaptivePortalSession", temptable)
        end    
    end

    -- Now that all dependencies are resolved remove the user from DB.
	if (valid) then
	    -- Delete Row
	    valid = db.delete("USERDBUsers", inTable)
	    if (not valid) then
	        break
	    end
	end
    end--for the Users

    -- return
    if (valid) then
        db.commitTransaction(true)
        return "OK", "STATUS_OK"
    else
        db.rollback()
        return "ERROR", "SSLVPN_GROUP_HAS_USER_DEL_FAILED"
    end
end

-- Login Policy Configuration
function sslAdmin.loginPolicyConfig (inputTable, rowid, operation)
    -- validate
    -- if not allowed to edit
    if (ACCESS_LEVEL ~= 0) then
        return "ACCESS_DENIED", "ADMIN_REQD"
    end
    db.beginTransaction() --begin transaction
    local valid = false

    if (db.typeAndRangeValidate(inputTable)) then
        if (operation == "add") then
            valid = db.insert(DBTable, inputTable)
        elseif (operation == "edit") then
            valid = db.update(DBTable, inputTable, rowid)
        elseif (operation == "delete") then
            valid = db.delete(DBTable, inputTable)
        end
    end
    
    -- return
    if (valid) then
        db.commitTransaction(true)
        return "OK", "STATUS_OK"            
    else
        db.rollback()
        return "ERROR", "SSLVPN_USER_CONFIG_FAILED"
    end

end

--[[
function : sslAdmin.UsersPerDomain
parameter:  sslAdmin.UsersPerDomain(groupName, maxAllowedUserName)
purpose: calculates number of users per domain
returns: ERROR or OK
--]]

function sslAdmin.UsersPerDomain (groupName, maxUsers)
	local numberOfUsersPerDomain = 0
	local i = 0
	local MAX_USERS = maxUsers
	local GroupsInADomain = {}
	local domainName = db.getAttribute("USERDBGroups", "GroupName", groupName, "DomainName")
	if(domainName ~= nil) then
		GroupsInADomain = db.getRows("USERDBGroups", "DomainName", domainName)
		for k,v in pairs (GroupsInADomain) do
			i = i + 1
			local row = GroupsInADomain[i]
			local noOfUsersPerGroup = db.getRows("USERDBUsers", "GroupName", row["USERDBGroups.GroupName"])
			if(noOfUsersPerGroup ~= nil ) then			
				numberOfUsersPerDomain = numberOfUsersPerDomain + tonumber(#noOfUsersPerGroup)		
			end			
			if(numberOfUsersPerDomain >= MAX_USERS) then
				return "ERROR", "USER_NUMBER_EXCEEDED"
			end
		end
		return "OK", "STATUS_OK"
	end
end

-- Domain local authentication config
function sslAdmin.localAuthdomainConfig (inputTable, rowid, operation)
    -- if not allowed to edit
    if (ACCESS_LEVEL ~= 0) then
    	return "ACCESS_DENIED", "ADMIN_REQD"
    end
    db.beginTransaction() --begin transaction
    local valid = false
    local inTable = {}
    inTable["SSLVPNUserLoginLocalDbStatus.LocalDbDisable"] = inputTable["SSLVPNUserLoginLocalDbStatus.LocalDbDisable"]
    if (operation == "add") then
        valid = false
    elseif (operation == "edit") then
        local domainTbl = db.getTable("USERDBDomains")                   
        local i = 0
        local flag = 0
        for k,v in pairs(domainTbl) do
            i = i + 1
            local row = domainTbl[i]
            if(row ~= nil) then
                if(row["USERDBDomains.AuthenticationType"] ~= "local") then
                    local custQuery = "SELECT * FROM USERDBGroups WHERE DomainName='" .. row["USERDBDomains.DomainName"] .. "'"
                    local table = db.getTable ("USERDBGroups", true, custQuery)
                    local s = 0
                	for l,m in pairs(table) do
                	    s = s + 1
                	    local groupRow = table[s]
                	    local groupName = groupRow["USERDBGroups.GroupName"]
                	    local userType = db.getAttribute("USERDBUsers", "GroupName", groupName, "UserType")
                	    if(userType == "3") then
                            flag = 1
                            break
                        else
                            flag = 0
                        end
                	end
                    if (flag == 1) then
                        break
                    end
                end
            end
        end
        if(inputTable["SSLVPNUserLoginLocalDbStatus.LocalDbDisable"] == "1") then
	        if(flag == 1) then
			    valid = db.update("SSLVPNUserLoginLocalDbStatus", inTable, rowid)
			else
                db.rollback()
			    return "ERROR","NO_ADMIN_PRESENT_IN_EXTERNAL_DOMAIN"
			end	
		else
		    valid = db.update("SSLVPNUserLoginLocalDbStatus", inTable, rowid)
		end     
    elseif (operation == "delete") then
            valid = false
    end

    -- return
    if (valid) then
        db.commitTransaction(true)
        return "OK", "STATUS_OK"
    else
        db.rollback()
        return "ERROR", "SSL_DOMAIN_LOCALAUTH_CONFIG_FAILED"
    end
end
