--[[
#### Adarsh Uppula
#### TeamF1
#### www.TeamF1.com
#### June 29, 2007

#### File: time.lua
#### Description: time functions

#### Revisions:
01b 04Sep12 cg,  Added validations for ntp sync through vpn
01a 04Jun09 shv, Added function to configure manual time settings.
]]--


--************* Requires *************

require "timeLib"

--************* Initial Code *************

--package time
time = {}

--table packages
ntp = {}

--************* Functions *************

-- time setTimeAndDate
function time.setTimeAndDate(inputTable)
	-- if not allowed to edit
	if (ACCESS_LEVEL ~= 0) then
		return "ACCESS_DENIED", "ADMIN_REQD"
	end
	if (db.typeAndRangeValidate(inputTable)) then
		--call C function
		local result = timeLib.setTimeAndDate(inputTable["ManualDate.Month"], inputTable["ManualDate.Day"], inputTable["ManualDate.Year"], inputTable["ManualDate.hours"], inputTable["ManualDate.minutes"], inputTable["ManualDate.seconds"])
		return "OK", "STATUS_OK"
	else
		return "ERROR", "TIME_STATUS_TIMEDATE_FAILED"
	end
end

-- time config
function time.ntp_config (inputTable, rowid, operation)
	-- if not allowed to edit
	if (ACCESS_LEVEL ~= 0) then
		return "ACCESS_DENIED", "ADMIN_REQD"
	end
	db.beginTransaction() --begin transaction
	local valid = false
	local status = -1
    local errStr

    if(inputTable["ntp.ntpMode"] == "0" and inputTable["ntp.setManualDate"] == "1") then
        valid = time.setTimeAndDate(inputTable)
        if (db.existsRow ("ManualDate", "_ROWID_", rowid)) then
            valid = db.update("ManualDate",inputTable,rowid)
        else
            valid = db.insert ("ManualDate",inputTable)
        end
        inputTable["ntp.setManualDate"] = "0"
    end

    -- if it sync over vpn policy then validate the vpn policy selected
    if (inputTable["ntp.ntpMode"] == "2") then
        local server1 = inputTable["ntp.server1"]
        local server2 = inputTable["ntp.server2"]
        if (inputTable ["ntp.useDefServers"] == "1") then
            if (inputTable ["ntp.networkMode"] == "6") then
                server1 = db.getAttribute ("ntpDefServers", "_ROWID_", "3", "serverName")
                server2 = db.getAttribute ("ntpDefServers", "_ROWID_", "4", "serverName")
            else
                server1 = db.getAttribute ("ntpDefServers", "_ROWID_", "1", "serverName")
                server2 = db.getAttribute ("ntpDefServers", "_ROWID_", "2", "serverName")
            end
        end
        status, errStr = timeLib.vpnServerValidate (inputTable["ntp.networkMode"], server1, server2, inputTable["ntp.VPNPolicy"])
    else
        status = 0
    end

    if (status == 0) then
        valid = true
    end

	-- config users
    if (valid) then
        valid = ntp.config(inputTable, rowid, operation)
        errStr = "TIME_STATUS_NTP_FAILED"
    end

	-- return
	if (valid) then
		db.commitTransaction(true)
		return "OK", "STATUS_OK"
	else
		db.rollback()
		return "ERROR", errStr
	end
end

-- ntp config
function ntp.config (inputTable, rowid, operation)
	-- validate
	if (ntp.inputvalidate(inputTable, operation)) then
		if (operation == "add") then
			return db.insert("ntp", inputTable)
		elseif (operation == "edit") then
			return db.update("ntp", inputTable, rowid)
		elseif (operation == "delete") then
			return db.delete("ntp", inputTable)
		end
	end
	return false
end

-- ntp inputvalidate
function ntp.inputvalidate (inputTable, operation)
	if (true) then
		return db.typeAndRangeValidate(inputTable)
	end
	return false
end

-- time config
function time.manualTimeConfig (inputTable, rowid, operation)
	-- if not allowed to edit
	if (ACCESS_LEVEL ~= 0) then
		return "ACCESS_DENIED", "ADMIN_REQD"
	end

    db.beginTransaction() --begin transaction
	local valid = false
	
	-- time config
	if (db.typeAndRangeValidate(inputTable)) then
		if (operation == "add") then
			valid = db.insert(DBTable, inputTable)
		elseif (operation == "edit") then
			valid = db.update(DBTable, inputTable, rowid)
		elseif (operation == "delete") then
			valid = db.delete(DBTable, inputTable)
		end
	end
	
	-- return
	if (valid) then
		db.commitTransaction(true)
		return "OK", "STATUS_OK"
	else
		db.rollback()
		return "ERROR", "MANUAL_TIME_CONFIG_FAILED"
	end
end
