--[[
--
--
-- Modification History
-- 01Mar13,ksr fixed SPR 36900
#### Hussain Vali.
#### TeamF1
#### www.TeamF1.com
#### July 25, 2008
#### File: vlanConfig.lua
#### Description: vlan configuration functions
#### Revisions:
]]--

require "teamf1lualib/ifDev"
--************* Initial Code *************
--table packages2
vlanConfig = {}

--************* Functions *************
-- vlan config
function vlanConfig.config (inputTable, rowid, operation)
    -- validate
    if (db.typeAndRangeValidate(inputTable)) then
        if (operation == "add") then
            return db.insert(DBTable, inputTable)
        elseif (operation == "edit") then
            return db.update(DBTable, inputTable, rowid)
        elseif (operation == "delete") then
            return db.delete(DBTable, inputTable)
        end
    end
    return false
end

-- check if disabling vlan is allowed.
function vlanConfig.disableAllow (inputTable, rowid, operation)
	local lanIf = "LAN" --db.getAttribute("networkInterface","LogicalIfName","LAN","VirtualIfName") or ''
	local wan1If = "WAN1" --db.getAttribute("networkInterface","LogicalIfName","WAN1","VirtualIfName") or ''
	local wan2If = "WAN2" --db.getAttribute("networkInterface","LogicalIfName","WAN2","VirtualIfName") or ''
	local message = ""
	local allow = 1
	local tcpDumpRow = db.getRow ("tcpdump", "tcpdumpEnabled", "1")
	if (tcpDumpRow ~= nil ) then
		if (tcpDumpRow ["tcpdump.LogicalIfName"] ~= lanIf or
            tcpDumpRow ["tcpdump.LogicalIfName"] ~= wan1If or
            tcpDumpRow ["tcpdump.LogicalIfName"] ~= wan2If) then
			message = message .. "Packet Capture, "
			allow = 0
		end
	end
	local igmpRow = db.getRow ("igmp", "IgmpEnable", "1")
	if (igmpRow ~= nil ) then
		if (igmpRow["igmp.igmpLogicalIfName"] ~= "LAN") then
			message = message .. "IGMP Proxy, "
			allow = 0
		end
	end
	local upnpRow = db.getRow ("upnp", "upnpEnable", "1")
	if (upnpRow ~= nil ) then
		if (upnpRow["upnp.interfaceName"] ~= "eth0.1") then
			message = message .. "UPnP, "
			allow = 0
		end
	end
	if (allow == 0) then
		return "ERROR", message
	else
		return "OK", message
	end
end

-- vlan status config
function vlanConfig.vlanStatusConfig (inputTable, rowid, operation)
	-- if not allowed to edit
	if (ACCESS_LEVEL ~= 0) then
		return "ACCESS_DENIED", "ADMIN_REQD"
	end

	local allowed = "OK"
	allowed, message  = vlanConfig.disableAllow (inputTable, rowid, operation)
	if (allowed == "ERROR" ) then
		message = "Please disable ".. message .. "on vlan enabled LANs."
		return "ERROR", message 
	end

	db.beginTransaction() --begin transaction
	local valid = false
	
	valid = vlanConfig.config (inputTable, rowid, operation)

    if(inputTable["VlanStatus.VlanState"] == "1" ) then
		--we need to revert dhcp to previous state
		local dhcpTable = db.getTable("dhcp")
		for i = 2,#(dhcpTable) do
			row = dhcpTable[i]
			row["dhcp.dhcpEnabled"] = row["dhcp.oldState"]
			DBTable = "dhcp"
			valid = vlanConfig.config(row, i, "edit");
		end
	else
		--we are here to disable dhcps
		local dhcpTable = db.getTable("dhcp")
		for i = 2,#(dhcpTable) do
			row = dhcpTable[i]
			local oldState = row["dhcp.dhcpEnabled"]
			row["dhcp.oldState"] = oldState
			row["dhcp.dhcpEnabled"] = "0"
			DBTable = "dhcp"
			valid = vlanConfig.config(row, i, "edit");
		end
	end

	-- return
	if (valid) then
		db.commitTransaction(true)    
        return "OK", "STATUS_OK"
	else
		db.rollback()
		return "ERROR", "VLAN_STATUS_CONFIG_FAILED"
	end
end

--vlan subnetting config
function vlanConfig.vlanSubnettingConfig (inputTable, rowid, operation)
	-- if not allowed to edit
	if (ACCESS_LEVEL ~= 0) then
		return "ACCESS_DENIED", "ADMIN_REQD"
	end

	db.beginTransaction() --begin transaction
	local valid = false
	
	valid = vlanConfig.config (inputTable, rowid, operation)
	
	-- return
	if (valid) then
		db.commitTransaction(true)    
        return "OK", "STATUS_OK"
	else
		db.rollback()
		return "ERROR", "VLAN_SUBNETTING_CONFIG_FAILED"
	end
end

function vlanConfig.configDep (inputTable, rowid, operation)    
	local valid1 = false

	if(operation == "delete")then
		for k,v in pairs(inputTable) do
			local vlanId = v
			valid1 = db.deleteRow("ifStatic","LogicalIfName","LAN"..vlanId)
			--valid1 = valid1 and db.deleteRow("dhcp","logicalIfName","LAN"..vlanId)
			valid1 = valid1 and db.setAttribute("dhcp","logicalIfName","LAN"..vlanId,"dhcpEnabled","0")
			valid1 = valid1 and db.setAttribute("dhcp","logicalIfName","LAN"..vlanId,"enableDnsProxy","0")
			valid1 = valid1 and db.deleteRow("DhcpRelay","logicalIfName","LAN"..vlanId)
			valid1 = valid1 and db.deleteRow("interVLANRouting","vlanId",vlanId)
			valid1 = valid1 and db.deleteRow("bridgeTable","bridgeIndex",vlanId)            
			valid1 = valid1 and db.deleteRow("tcpdump","LogicalIfName","LAN"..vlanId)
			valid1 = valid1 and db.deleteRow("zones","ZoneLogicalIfName","LAN"..vlanId)
			valid1 = valid1 and db.deleteRow("DhcpRelay","LogicalIfName","LAN"..vlanId)
		end
	elseif(operation == "add") then
    	local ifStatic = {}
	    ifStatic["ifStatic.LogicalIfName"] = "LAN" .. inputTable["vlan.vlanId"]
	    ifStatic["ifStatic.AddressFamily"] = "2"
	    ifStatic["ifStatic.StaticIp"] = inputTable["ifStatic.StaticIp"]
	    ifStatic["ifStatic.NetMask"] = inputTable["ifStatic.NetMask"]
	    ifStatic["ifStatic.Gateway"] = "0.0.0.0"
	    DBTable = "ifStatic"
	    valid1 = vlanConfig.config(ifStatic, "-1", "add");		
        if (valid1) then
			local dhcp = {}			
		if (operation == "add") then
			local dhcpRow = db.getRow("dhcp","logicalIfName","LAN".. inputTable["vlan.vlanId"])			
			if (dhcpRow == nil) then					
        			dhcp["dhcp.logicalIfName"] = "LAN" .. inputTable["vlan.vlanId"]
				    dhcp["dhcp.domainName"] = inputTable["dhcp.domainName"]
   	    			dhcp["dhcp.startAddress"] = inputTable["dhcp.startAddress"]
            	    dhcp["dhcp.endAddress"] = inputTable["dhcp.endAddress"]
            	    dhcp["dhcp.primaryDnsServer"] = inputTable["dhcp.primaryDnsServer"]
            	    dhcp["dhcp.secondaryDnsServer"] = inputTable["dhcp.secondaryDnsServer"]
            	    dhcp["dhcp.winsServer"] = inputTable["dhcp.winsServer"]
            	    dhcp["dhcp.enableArpBcast"] = "0"            	   
            	    if(inputTable["dhcp.dhcpEnabled"] == "2") then
            	        dhcp["dhcp.dhcpEnabled"] = "1"
            	    else
            	        dhcp["dhcp.dhcpEnabled"] = "0"
            	    end
   		    		
            		dhcp["dhcp.leaseTime"] = inputTable["dhcp.leaseTime"]
   		    		dhcp["dhcp.enableDnsProxy"] = inputTable["dhcp.enableDnsProxy"]
            		DBTable = "dhcp"
            		valid1 = vlanConfig.config(dhcp, "-1", "add");
			else				
				valid1 = valid1 and db.setAttribute("dhcp","logicalIfName","LAN"..inputTable["vlan.vlanId"],"dhcpEnabled","1")
				valid1 = valid1 and db.setAttribute("dhcp","logicalIfName","LAN"..inputTable["vlan.vlanId"],"enableDnsProxy","1")
			end
		end
        end

        if(valid1) then
            local DhcpRelay = {}
            DhcpRelay["DhcpRelay.LogicalIfName"] = "LAN" .. inputTable["vlan.vlanId"]
            DhcpRelay["DhcpRelay.virtualIfName"] = "eth0." .. inputTable["vlan.vlanId"]
            if(inputTable["dhcp.dhcpEnabled"] == "1") then
                DhcpRelay["DhcpRelay.DhcpRelayStatus"] = "1"
            else
                DhcpRelay["DhcpRelay.DhcpRelayStatus"] = "0"
            end
            if(inputTable["dhcp.RelayGateway"] ~= nil) then
                DhcpRelay["DhcpRelay.RelayGateway"] = inputTable["dhcp.RelayGateway"]
            else
                DhcpRelay["DhcpRelay.RelayGateway"] = ""
            end
            DBTable = "DhcpRelay"
            valid1 = vlanConfig.config(DhcpRelay,"-1", "add");
        end

        if(valid1) then
	        local zones = {}
        	zones["zones.ZoneLogicalIfName"] = "LAN" .. inputTable["vlan.vlanId"]
	        zones["zones.ZoneType"] = "SECURE"
        	DBTable = "zones"
	        valid1 = vlanConfig.config(zones,"-1", "add");
        end      

        if(valid1) then
            local tcpdump = {}
            tcpdump["tcpdump.LogicalIfName"] = "LAN" .. inputTable["vlan.vlanId"]
            tcpdump["tcpdump.tcpdumpEnabled"] = "0"
            DBTable = "tcpdump"
            valid1 = vlanConfig.config(tcpdump,"-1", "add");
        end

        if(valid1) then
			local interVlan = {}
			interVlan["interVLANRouting.vlanId"] = inputTable["vlan.vlanId"]
			interVlan["interVLANRouting.status"] = inputTable["interVLANRouting.status"];
            interVlan["interVLANRouting.ifName"] = "eth0." .. inputTable["vlan.vlanId"];
			DBTable = "interVLANRouting"
			valid1 = vlanConfig.config(interVlan,"-1","add");
		end
    end

	return valid1 or true
end

-- vlan config
function vlanConfig.vlanConfig (inputTable, rowid, operation)	
	local valid = false	
	if (operation == "add" and inputTable["interVLANRouting.status"] ~= nil and inputTable["insertRpt"] ~= "1") then
        DBTable = "vlan"
    end
    valid = db.insert("vlan", inputTable)   
	-- return
	if (valid) then		
        return "OK", "STATUS_OK"
	else		
		return "ERROR", "PORT_VLAN_CONFIG_FAILED"
	end
end

-- vlan config
function vlanConfig.portVlanConfig (inputTable,rowid, commitCheck)
	-- if not allowed to edit
	if (ACCESS_LEVEL ~= 0) then
		return "ACCESS_DENIED", "ADMIN_REQD"
	end

    -- temp, we need to reuse this foo under single commit 
    if (commitCheck ~= 1) then
    	db.beginTransaction() --begin transaction
    end
	local valid = false
    local isUpdated = false
	
    local portLogName = "vlan." .. db.getAttribute ("VlanPortMgmt", "_ROWID_", rowid, "PortLogicalName")
    local defPVID = db.getAttribute ("VlanPortMgmt", "_ROWID_", rowid, "DefaultPVID")
    local table = db.getTable ("vlan")
    local i = 0
	for k,v in pairs(table) do
	    i = i + 1
        local row = table[i]
        local reqUpdate = false
        if (row ~= nil) then
            if (inputTable["vlan.row_" .. row["vlan._ROWID_"]] == nil) then
                if (row["vlan.portName"] ~= nil and "vlan." .. row["vlan.portName"] == portLogName) then
                    -- in any case, the current PVID must be mapped on the membership table ??
                    if (defPVID ~= row["vlan.vlanId"]) then
                        valid = vlanConfig.config ({row["vlan._ROWID_"]}, -1, "delete") 
                    else
                        valid = true
                    end
                    isUpdated = true
                    if (not valid) then
                        break
                    end
                end                
            end
        end
    end

    i = 0
	for k,v in pairs(table) do
	    i = i + 1
        local row = table[i]
        local reqUpdate = false
        if (row ~= nil) then
            if (inputTable["vlan.new_vlanId_" .. row["vlan.vlanId"]] ~= nil) then
                local tab = {}
                tab ["vlan.vlanId"] = inputTable["vlan.new_vlanId_" .. row["vlan.vlanId"]]
                tab ["vlan.vlanName"] = db.getAttribute ("vlan", "vlanId", tab ["vlan.vlanId"], "vlanName");
                tab ["vlan.portName"] = db.getAttribute ("VlanPortMgmt", "_ROWID_", rowid, "PortLogicalName")
                vlanConfig.config (tab, -1, "add")
            end
        end
    end

    if (not isUpdated) then
        valid = true
    end

    -- temp, we need to reuse this foo under single commit 
    if (commitCheck == 1) then
        return valid;
    end

	-- return
	if (valid) then
		db.commitTransaction(true)    
        return "OK", "STATUS_OK"
	else
		db.rollback()
		return "ERROR", "PORT_VLAN_CONFIG_FAILED"
	end
end

-- vlan port mgmt update
function vlanConfig.portMgmtUpdate (inputTable)
	-- if not allowed to edit
	if (ACCESS_LEVEL ~= 0) then
		return "ACCESS_DENIED", "ADMIN_REQD"
	end

	db.beginTransaction() --begin transaction
	local valid = false
	
    local boardName = db.getAttribute ("unitName","_ROWID_","1","unitName")
    local portVlanConfig = db.getRowWhere ("VlanPortMgmt","PortName = 'OptionalPort'")
    if (portVlanConfig ~= nil) then
        if (inputTable["ConfigPort.LogicalIfName"] == "LAN") then
            
            if (boardName ~= nil and boardName ~= "WRVS5110") then
                portVlanConfig["VlanPortMgmt.PortType"] = inputTable["ConfigPort.LogicalIfName"]
                portVlanConfig["VlanPortMgmt.DefaultPVID"] = "1"
            else
                portVlanConfig["VlanPortMgmt.PortType"] = "BRIDGE"
            end
        else
            if (inputTable["ConfigPort.LogicalIfName"] == "WAN2") then
                portVlanConfig["VlanPortMgmt.PortType"] = "WAN"
            else
                portVlanConfig["VlanPortMgmt.PortType"] = inputTable["ConfigPort.LogicalIfName"]
            end
            if (boardName ~= nil and boardName ~= "WRVS5110") then
                portVlanConfig["VlanPortMgmt.DefaultPVID"] = "4094"
            else
                portVlanConfig["VlanPortMgmt.DefaultPVID"] = "1"
            end
                portVlanConfig["VlanPortMgmt.PortVlanMode"] = "0"            
                local updateError = false
                local portLogName = "vlan." .. portVlanConfig["VlanPortMgmt.PortLogicalName"]
                local table = db.getTable ("vlan")
                local i = 0
            	for k,v in pairs(table) do
	                i = i + 1
                    local row = table[i]
                    if (row["vlan.portName"] == portVlanConfig["VlanPortMgmt.PortLogicalName"]) then
                        valid =  db.delete ("vlan",{row["vlan._ROWID_"]})
                        if (not valid) then
                            updateError = true
                            break
                        end
                    end
                end
                if (updateError) then
                    db.rollback()
		            return "ERROR", "VLAN_UPDATE_CONFIG_FAILED"
                end
           end
        valid = db.update("VlanPortMgmt", portVlanConfig, portVlanConfig["VlanPortMgmt._ROWID_"])
        if (valid and (portVlanConfig["VlanPortMgmt.PortType"] == "LAN" or portVlanConfig["VlanPortMgmt.PortType"] == "BRIDGE") and boardName ~= nil) then
            local portLogName = "vlan." .. portVlanConfig["VlanPortMgmt.PortLogicalName"]
            local vlanConfigRow = db.getRow ("vlan","vlanId","1")
            if (vlanConfigRow ~= nil) then
                locTable = {}
                locTable ["vlan.vlanId"] = 1
                locTable ["vlan.vlanName"] = db.getAttribute ("vlan", "vlanId", "1", "vlanName")
                locTable ["vlan.portName"] = portVlanConfig["VlanPortMgmt.PortLogicalName"]
                valid =  db.insert ("vlan", locTable)
            end
            if (not valid) then
                db.rollback()
		        return "ERROR", "VLAN_UPDATE_CONFIG_FAILED"
            end
        end
    else
        valid = true        
    end

	-- return
	if (valid) then
		db.commitTransaction(true)    
        return "OK", "STATUS_OK"
	else
		db.rollback()
		return "ERROR", "PORT_VLAN_CONFIG_FAILED"
	end
end

function vlanConfig.vlanDeleteCheck (rows)
    for k,v in pairs(rows) do
        local row = db.getRow ("vlan","vlanId",v)
        local usingRows = db.getRowsWhere ("TrafficSelector","MatchType = 4 and VlanId = " .. row["vlan.vlanId"])
        if (usingRows ~= nil and #usingRows ~= 0) then
            return "ERROR", "VLAN_IN_USE_BY_BW_TRAFFIC_SELECTOR"
        end
    end
    return "OK", "STATUS_OK"
end

function vlanConfig.portVlanEditCheck (rowid)
    local portVlanConfig = db.getRow ("VlanPortMgmt","_ROWID_",rowid)
    local bwStatus = db.getAttribute ("BandwidthProfileStatus","_ROWID_","1","Status")
	if (portVlanConfig ~= nil) then
	local usingRows = db.getRowsWhere ("TrafficSelector","MatchType = 3 and PortName = '" .. portVlanConfig["VlanPortMgmt.PortName"] .. "'")
		if (usingRows ~= nil and #usingRows ~= 0) then
			if(bwStatus ~= "0") then 
			return "ERROR", "PORT_VLAN_IN_USE_BY_BW_TRAFFIC_SELECTOR"
			end
		end
	end
  
    return "OK", "STATUS_OK"
end

--[[
******************************************************************** 
* vlanConfig.vlanAddWr  - wrapper function to call API to do deifferent  
* operation
* RETURNS: 'OK' or 'ERROR' 
*/
--]]

function vlanConfig.vlanAddWr (inputTable, rowId, operation)
    
    -- if not allowed to edit
	if (ACCESS_LEVEL ~= 0) then
		return "ACCESS_DENIED", "ADMIN_REQD"
	end
	
	-- Collect IP address
	if(inputTable["ifStatic.IPByte1"] ~= nil) then
        inputTable["ifStatic.StaticIp"] = util.joinFields(inputTable,"ifStatic","IPByte", ".",4)
    else
    	inputTable["ifStatic.StaticIp"] = ''
    end
    
    -- Collect Subnet mask address
    
    if(inputTable["ifStatic.SMByte1"] ~= nil) then
        inputTable["ifStatic.NetMask"] = util.joinFields(inputTable,"ifStatic","SMByte", ".",4)
    else
    	inputTable["ifStatic.NetMask"] = ''
    end
    
    -- Collect Start Dhcp Ip address
    if(inputTable["dhcp.startAddress1"] ~= nil) then
        inputTable["dhcp.startAddress"] = util.joinFields(inputTable,"dhcp","startAddress", ".",4)
    else
    	inputTable["dhcp.startAddress"] = ''
    end
    
    -- Collect End Dhcp Ip address
    
    if(inputTable["dhcp.endAddress1"] ~= nil) then
        inputTable["dhcp.endAddress"] = util.joinFields(inputTable,"dhcp","endAddress", ".",4)
    else
    	inputTable["dhcp.endAddress"] = ''
    end
    
    -- Collect Primary Dns Ip address
    
    if(inputTable["dhcp.primaryDnsServer1"] ~= nil) then
        inputTable["dhcp.primaryDnsServer"] = util.joinFields(inputTable,"dhcp","primaryDnsServer", ".",4)
    else
    	inputTable["dhcp.primaryDnsServer"] = ''
    end
    
    -- Collect Secondary Dns Ip address
    
    if(inputTable["dhcp.secondaryDnsServer1"] ~= nil) then
        inputTable["dhcp.secondaryDnsServer"] = util.joinFields(inputTable,"dhcp","secondaryDnsServer", ".",4)
    else
    	inputTable["dhcp.secondaryDnsServer"] = ''
    end
    
    -- Collect Win Server Ip address
    
    if(inputTable["dhcp.winsServer1"] ~= nil) then
        inputTable["dhcp.winsServer"] = util.joinFields(inputTable,"dhcp","winsServer", ".",4)
    else
    	inputTable["dhcp.winsServer"] = ''
    end
    
    -- Collect relay Ip address
    
    if(inputTable["dhcp.RelayGateway1"] ~= nil) then
        inputTable["dhcp.RelayGateway"] = util.joinFields(inputTable,"dhcp","RelayGateway", ".",4)
    else
    	inputTable["dhcp.RelayGateway"] = ''
    end
	
    -- Configure vlanName, vlanId, portname
    --automation part begin	
    local tableIndex = 1
    local errorFlag = "OK"
    if(errorFlag == "OK" or errorFlag == 0) then
    if(operation == "add") then
	    validationTable = {
            function() return validations.rangeValidate(2, 4093, inputTable["vlan.vlanId"],"","Invalid Number") end,
           -- function() return validations.ipAddressValidate(inputTable["ifStatic.StaticIp"],"","IP Address error") end,
           -- function() return validations.subnetCheck(inputTable["ifStatic.NetMask"],"","Subnet Mask Error") end,
            function() return validations.checkboxValidate(inputTable["dhcp.dhcpEnabled"],"0",8) end,
            function() return validations.checkboxValidate(inputTable["dhcp.dhcpEnabled"],"1",5) end,
	    function() return validations.ipRangeValidate(inputTable["dhcp.startAddress"],inputTable["dhcp.endAddress"],"DHCP Start/Stop Error") end,
	    function() return validations.ipAddressValidate(inputTable["dhcp.primaryDnsServer"],"","Primary DNS Error") end,
	    function() return validations.ipAddressValidate(inputTable["dhcp.secondaryDnsServer"],"","Secondary DNS Error") end,
	    function() return validations.ipAddressValidate(inputTable["dhcp.winsServer"],"","WINS server error") end,
	    function() return validations.leaseTimeValidate(inputTable["dhcp.leaseTime"],"Lease Time Error") end,
	    function() return validations.checkboxValidate(inputTable["dhcp.dhcpEnabled"],"2",1) end,		
	    function() return validations.ipAddressValidate(inputTable["DhcpRelay.RelayGateway"],"","Relay Gateway Error") end	      
         }
     else
        validationTable = {           
            -- function() return validations.ipAddressValidate(inputTable["ifStatic.StaticIp"],"","IP Address error") end,
            -- function() return validations.subnetCheck(inputTable["ifStatic.NetMask"],"","Subnet Mask Error") end,
            function() return validations.checkboxValidate(inputTable["dhcp.dhcpEnabled"],"0",8) end,
            function() return validations.checkboxValidate(inputTable["dhcp.dhcpEnabled"],"1",5) end,
	    function() return validations.ipRangeValidate(inputTable["dhcp.startAddress"],inputTable["dhcp.endAddress"],"DHCP Start/Stop Error") end,
	    function() return validations.ipAddressValidate(inputTable["dhcp.primaryDnsServer"],"","Primary DNS Error") end,
	    function() return validations.ipAddressValidate(inputTable["dhcp.secondaryDnsServer"],"","Secondary DNS Error") end,
	    function() return validations.ipAddressValidate(inputTable["dhcp.winsServer"],"","WINS server error") end,
	    function() return validations.leaseTimeValidate(inputTable["dhcp.leaseTime"],"Lease Time Error") end,
	    function() return validations.checkboxValidate(inputTable["dhcp.dhcpEnabled"],"2",1) end,		
	    function() return validations.ipAddressValidate(inputTable["DhcpRelay.RelayGateway"],"","Relay Gateway Error") end	      
         }
      end
     
         while (tableIndex <= #(validationTable)) do
            validationFunc = validationTable[tableIndex]
            errorFlag, statusCode,field,skipValidations = validationFunc()
            if(errorFlag==1 or errorFlag == "ERROR") then
                statusMessage = db.getAttribute("stringsMap", "stringId", statusCode, LANGUAGE) or statusCode
                statusMessage = field .. ": " .. statusMessage
                break
            end 
            if(skipValidations ~= "") then
                tableIndex = tableIndex + skipValidations + 1
            else
                tableIndex = tableIndex + 1
            end
        end
    end
    
    -- If vlan Id value is between 2-4093 then 
    -- configure vlanId otherwise return error 
     if(errorFlag == "ERROR" or errorFlag == 1) then
        return errorFlag, statusMessage
     else  	
         if (inputTable ["dhcp.dhcpEnabled"] == "2") then
             errorFlag  = "OK"
             errorFlag = ifDev.ipSubnet (inputTable["ifStatic.StaticIp"],inputTable["ifStatic.NetMask"],inputTable["dhcp.startAddress"],inputTable["dhcp.endAddress"])
             if(errorFlag ~= "OK") then
                 statusCode = errorFlag
                 errorFlag = "ERROR"
                 return errorFlag, statusCode
             end
         end
    if (operation == "add") then
        if(db.existsRow("vlan", "vlanName", inputTable["vlan.vlanName"])) then
           errorFlag, statusCode = "ERROR", "VLAN_NAME_ALREADY_EXISTED" 
        end
        local countVlanMacOffSet = 0
	    local vlanUniMacMaxCount = db.getAttribute("vlanMac", "_ROWID_", "1","vlanUniqueMacMaxCount") or 0
	    local vLanMacOffset = db.getAttribute("vlanMac", "_ROWID_", "1","vlanLanMacOffset") or 0
	    vLanMacOffset = tonumber(vLanMacOffset) + 3
	    vlanUniMacMaxCount = (vLanMacOffset + tonumber(vlanUniMacMaxCount)) - 1
        local vlanMacOffSetType = db.getAttribute("vlanMac", "_ROWID_", "1","vlanMacOffsetType") or ''	    	
	    if(vlanMacOffSetType == "0") then
	        countVlanMacOffSet = 0
	    else
	        countVlanMacOffSet = vlanConfig.vlanMacOffSetVal ()
	        if(countVlanMacOffSet == nil or (countVlanMacOffSet > vlanUniMacMaxCount)) then
	            countVlanMacOffSet = 0
	        end
	    end	    	     
        if(errorFlag == "OK" or errorFlag == 0) then
            inputTable_all = {}
            inputTable_all["vlan.vlanId"]= tonumber(inputTable["vlan.vlanId"]);
            inputTable_all["vlan.vlanName"]= inputTable["vlan.vlanName"];
            inputTable_all["vlan.portName"]= inputTable["vlan.portName"];
            inputTable_all["vlan.vlanProfState"]= "1"
            inputTable_all["vlan.vlanMacOffset"]= countVlanMacOffSet
            inputTable_all["interVLANRouting.status"]= inputTable["interVLANRouting.status"]
            inputTable_all["insertRpt"] = "0"
            db.beginTransaction ()
            errorFlag, statusCode = vlanConfig.vlanConfig(inputTable_all, "-1", "add") 
            
            inputTable_all = {}
            inputTable_all["vlan.vlanId"]= tonumber(inputTable["vlan.vlanId"]);
            inputTable_all["vlan.vlanName"]= inputTable["vlan.vlanName"];
            inputTable_all["vlan.portName"]= inputTable["vlan.portName2"];
            inputTable_all["vlan.vlanProfState"]= "1"
            inputTable_all["vlan.vlanMacOffset"]= countVlanMacOffSet
            inputTable_all["interVLANRouting.status"]= inputTable["interVLANRouting.status"]
            inputTable_all["insertRpt"] = "1"
            errorFlag, statusCode = vlanConfig.vlanConfig(inputTable_all, "-1", "add")
            if(errorFlag ~= "OK") then                   
                valid = false
            end
            -- Adding configuration for vlan subnetting.
            if(errorFlag == "OK") then
                valid = vlanConfig.configDep (inputTable, "-1", "add")
                DBTable = "vlan"	        
            end	    
        else
	        return errorFlag, statusCode
        end
    
        -- Add port membership 
    
        -- Check for port1
        
        local commonLanStr = "LAN"
        DBTable = "vlan"
        if(valid) then
            if(inputTable["vlan.port1"] == "1") then
        
                -- Add a row in vlan table with portname as port1
        
                local portTbl = {}
                portTbl["vlan.vlanId"] = tonumber(inputTable["vlan.vlanId"])
                portTbl["vlan.vlanName"] = inputTable["vlan.vlanName"];
                portTbl["vlan.portName"] = "port1"
                portTbl["vlan.vlanProfState"] = "1"
                valid = db.insert(DBTable, portTbl)
                if(not valid) then
                    db.rollback()
                    errorFlag, statusCode = "ERROR", "VLAN_CONFIG_FAILED"
                end         
            end
        end    
        if(valid) then
        -- Check for port2
    
        if(inputTable["vlan.port2"] == "1") then
        
            -- Add a row in vlan table with portname as port2
        
            local portTbl = {}
            portTbl["vlan.vlanId"] = tonumber(inputTable["vlan.vlanId"])
            portTbl["vlan.vlanName"] = inputTable["vlan.vlanName"];
            portTbl["vlan.portName"] = "port2"
            portTbl["vlan.vlanProfState"] = "1"
            valid = db.insert(DBTable, portTbl)
            if(not valid) then
                db.rollback()
                errorFlag, statusCode = "ERROR", "VLAN_CONFIG_FAILED"
            end
         
        end
        end
    
        if(valid) then
        -- Check for port3
    
        if(inputTable["vlan.port3"] == "1") then
        
            -- Add a row in vlan table with portname as port3
        
            local portTbl = {}
            portTbl["vlan.vlanId"] = tonumber(inputTable["vlan.vlanId"])
            portTbl["vlan.vlanName"] = inputTable["vlan.vlanName"];
            portTbl["vlan.portName"] = "port3"
            portTbl["vlan.vlanProfState"] = "1"
            valid = db.insert(DBTable, portTbl)
            if(not valid) then
                db.rollback()
                errorFlag, statusCode = "ERROR", "VLAN_CONFIG_FAILED"
            end
         
        end
        end
    
        if(valid) then
        -- Check for port4
    
        if(inputTable["vlan.port4"] == "1") then
        
            -- Add a row in vlan table with portname as port4
        
            local portTbl = {}
            portTbl["vlan.vlanId"] = tonumber(inputTable["vlan.vlanId"])
            portTbl["vlan.vlanName"] = inputTable["vlan.vlanName"];
            portTbl["vlan.portName"] = "port4"
            portTbl["vlan.vlanProfState"] = "1"
            valid = db.insert(DBTable, portTbl)
            if(not valid) then
                db.rollback()
                errorFlag, statusCode = "ERROR", "VLAN_CONFIG_FAILED"
            end
        end
        end

    
        -- Store LDAP information
    
        if(valid) then
            DBTable = "DhcpLdapServerInfo"
            local ldapTbl = {}
            ldapTbl["DhcpLdapServerInfo.DhcpLdapStatus"] = inputTable["DhcpLdapServerInfo.DhcpLdapStatus"]
            ldapTbl["DhcpLdapServerInfo.LogicalIfName"] = commonLanStr .. tonumber(inputTable["vlan.vlanId"])
            if(ldapTbl["DhcpLdapServerInfo.DhcpLdapStatus"] == "1") then
                if(inputTable["DhcpLdapServerInfo.DhcpLdapServerIP"] ~= nil) then
                    ldapTbl["DhcpLdapServerInfo.DhcpLdapServerIP"] = inputTable["DhcpLdapServerInfo.DhcpLdapServerIP"]
                end
                if(inputTable["DhcpLdapServerInfo.DhcpLdapServerPort"] ~= nil) then
                    ldapTbl["DhcpLdapServerInfo.DhcpLdapServerPort"] = inputTable["DhcpLdapServerInfo.DhcpLdapServerPort"]
                end
                if(inputTable["DhcpLdapServerInfo.DhcpLdapSearchBase"] ~= nil) then
                    ldapTbl["DhcpLdapServerInfo.DhcpLdapSearchBase"] = inputTable["DhcpLdapServerInfo.DhcpLdapSearchBase"]
                end
            end
            valid = db.insert (DBTable,ldapTbl)
            if(valid) then
                db.commitTransaction(true)
                errorFlag, statusCode = "OK", "STATUS_OK"
            else
                db.rollback()
                errorFlag, statusCode = "ERROR", "VLAN_CONFIG_FAILED"
            end
        end  
    
        return errorFlag, statusCode
    elseif (operation == "edit") then
        local commonLanStr = "LAN"
        errorFlag, statusCode = "OK", "STATUS_OK"
        local vlanRow = db.getRow ("vlan", "_ROWID_", rowId)
        local vlanId = vlanRow["vlan.vlanId"]
        local vlanId1 = vlanId
        -- Update interVlanRouting table
               
        local interVlan = db.getRow("interVLANRouting", "vlanId", vlanId)
        db.beginTransaction() --begin transaction
        valid = db.update ("interVLANRouting", inputTable, interVlan["interVLANRouting._ROWID_"])
        if(not valid) then
            db.rollback()
            errorFlag, statusCode = "ERROR", "VLAN_CONFIG_FAILED"            
        end        
        
        if(vlanId == "1") then vlanId = "" end
        
        -- update ifStatic Table
        
        if(valid) then
            local ifStaticRow = db.getRow("ifStatic", "logicalIfName", commonLanStr .. vlanId)           
            valid = db.update ("ifStatic", inputTable, ifStaticRow["ifStatic._ROWID_"])
            if(not valid) then
                db.rollback()
                errorFlag, statusCode = "ERROR", "VLAN_CONFIG_FAILED"
            end
        end
        
        -- Update port membership details
        
        if(valid) then
            
            -- As there are 8 ports take 8 loops
            
            local j = 1
            while j <= 8 do                
                valid = vlanConfig.addPortMemberShip (inputTable, vlanRow["vlan.vlanName"], vlanId1, j)
                if(not valid) then
                    db.rollback()
                    errorFlag, statusCode = "ERROR", "VLAN_CONFIG_FAILED"
                    break
                end
                j = j + 1
            end
        end       
        
        -- Update Dhcp table 
        
        -- store dhcp.dhcpEnabled value
        
        local dhcpEnableVal = inputTable["dhcp.dhcpEnabled"]
        
        if(valid) then 
            local dhcpRow = db.getRow("dhcp", "logicalIfName", commonLanStr .. vlanId)
            if(dhcpEnableVal == "2") then
                inputTable["dhcp.dhcpEnabled"] = "1"
            else
                inputTable["dhcp.dhcpEnabled"] = "0"
            end
            valid = db.update ("dhcp", inputTable, dhcpRow["dhcp._ROWID_"])
            if(not valid) then
                db.rollback()
                errorFlag, statusCode = "ERROR", "VLAN_CONFIG_FAILED"            
            end
        end
        
        -- update dhcp relay table
        
        if(valid) then
                    
            local dhcpRelay = db.getRow("DhcpRelay", "logicalIfName", commonLanStr .. vlanId)
            if(dhcpEnableVal == "1") then
                dhcpRelay["DhcpRelay.DhcpRelayStatus"] = "1"
                dhcpRelay["DhcpRelay.RelayGateway"] = inputTable["dhcp.RelayGateway"]
            else
                dhcpRelay["DhcpRelay.DhcpRelayStatus"] = "0"
            end
            valid = db.update ("DhcpRelay", dhcpRelay, dhcpRelay["DhcpRelay._ROWID_"])
            if(not valid) then
                db.rollback()
                errorFlag, statusCode = "ERROR", "VLAN_CONFIG_FAILED"                            
            end            
        end
        
        -- update DhcpLdapServerInfo table
        
        if(valid) then
                    
            local ldapServer = db.getRow("DhcpLdapServerInfo", "logicalIfName", commonLanStr .. vlanId)            
            valid = db.update ("DhcpLdapServerInfo", inputTable, ldapServer["DhcpLdapServerInfo._ROWID_"])
            if(not valid) then
                db.rollback()
                errorFlag, statusCode = "ERROR", "VLAN_CONFIG_FAILED"
            else
                db.commitTransaction(true)
                errorFlag, statusCode = "OK", "STATUS_OK"
            end            
        end                
        return errorFlag, statusCode
    end
end
end

--[[
******************************************************************** 
* vlanConfig.vlanDeleteWr - wrapper function to call API to do delete
* operation
* RETURNS: 'OK' or 'ERROR' 
*/
--]]

function vlanConfig.vlanDeleteWr (inputTable)
    -- Loop through the vlanIds received from the frontend
                
    	for k,v in pairs(inputTable) do

    	     --Checking for used vlans.

            local vlanId = tonumber(v)
            local vlanName = db.getAttribute ("vlan","vlanId",vlanId, "vlanName")
            --New Start
            local a = 0
            local s = 0
            -- New End
            if (vlanName) then                
                -- delete the entries for vlanId from vlan
                local rows = db.getRowsWhere ("vlan", "vlanId="..vlanId);
                for kk, vv in pairs (rows) do
                    valid = vlanConfig.config ({vv["vlan._ROWID_"]}, -1, "delete")
                    if(not valid) then
                        break
                    end
                end
            end
        end
        
        -- Delete rows from other tables
        if(valid) then
            for k,v in pairs(inputTable) do
			    local vlanId = v
			    valid = db.deleteRow("ifStatic","LogicalIfName","LAN"..vlanId)
			    valid = db.deleteRow("dhcp","LogicalIfName","LAN"..vlanId)		
			    -- valid = valid and db.setAttribute("dhcp","logicalIfName","LAN"..vlanId,"dhcpEnabled","0")
			    -- valid = valid and db.setAttribute("dhcp","logicalIfName","LAN"..vlanId,"enableDnsProxy","0")
                if(db.existsRow ("VlanPortMgmt", "DefaultPVID", vlanId)) then
                   valid = valid and db.setAttribute("VlanPortMgmt","DefaultPVID",vlanId,"oldVlanId","1")
                   valid = valid and db.setAttribute("VlanPortMgmt","DefaultPVID",vlanId,"DefaultPVID","1")
                end
			    valid = valid and db.deleteRow("DhcpRelay","logicalIfName","LAN"..vlanId)
			    valid = valid and db.deleteRow("interVLANRouting","vlanId",vlanId)
			    valid = valid and db.deleteRow("tcpdump","LogicalIfName","LAN"..vlanId)
			    valid = valid and db.deleteRow("zones","ZoneLogicalIfName","LAN"..vlanId)
			    valid = valid and db.deleteRow("DhcpRelay","LogicalIfName","LAN"..vlanId)
			    valid = valid and db.deleteRow("DhcpRelay","LogicalIfName","LAN"..vlanId)
			    valid = valid and db.deleteRow("DhcpLdapServerInfo","LogicalIfName","LAN"..vlanId)
		    end
		end
        
        if(valid) then
            return "OK", "STATUS_OK"
        else
            return "ERROR", "VLAN_CONFIG_IN_USE"
        end
end



--[[
******************************************************************** 
* vlanConfig.getVlanValues - Used to get all the values
* corresponding to a vlan to show to the user
* RETURNS: row of vlan values
*/
--]]

function vlanConfig.getVlanValues (rowid)
    
    -- get the row from vlan and based on vlanId get all the row from other tables
    
    local vlanRow = db.getRow("vlan","_ROWID_",rowid)
    local dmzEnable = db.getAttribute ("ConfigPort","_ROWID_","1","LogicalIfName")
    
    local vlanId = vlanRow["vlan.vlanId"]
    local vlanId1 = vlanId
    if (vlanId == "1") then vlanId = "" end
    
    local memberShipTbl = {}
    
    -- Find out if port membership exists for this vlan
	    
    if(db.existsRowWhere ("vlan", "portName='port1' and vlanId='" .. vlanId1 .."'")) then
        memberShipTbl["vlan.port1"] = "1"
    end

    if(db.existsRowWhere ("vlan", "portName='port2' and vlanId='" .. vlanId1 .."'")) then
        memberShipTbl["vlan.port2"] = "1"
    end

    if(db.existsRowWhere ("vlan", "portName='port3' and vlanId='" .. vlanId1 .."'")) then
        memberShipTbl["vlan.port3"] = "1"
    end  

    if(db.existsRowWhere ("vlan", "portName='port4' and vlanId='" .. vlanId1 .."'")) then
        if(dmzEnable ~= "LAN") then
            memberShipTbl["vlan.port4"] = "0"
        else
            memberShipTbl["vlan.port4"] = "1"
        end
    end
	
    -- append to vlanRow that we will return 
    
    vlanRow = util.tableAppend(vlanRow, memberShipTbl)
    
	local ip = db.getRow("ifStatic","LogicalIfName","LAN" .. vlanId) or {}
	
	if(ip["ifStatic.StaticIp"] ~= nil and ip["ifStatic.StaticIp"] ~= "") then
	    ip["ifStatic.StaticIp"] = util.split(ip["ifStatic.StaticIp"], ".")
	    
	    --begin: split ip address
	    
	    ip["ifStatic.IPByte1"] = ip["ifStatic.StaticIp"][1]
	    ip["ifStatic.IPByte2"] = ip["ifStatic.StaticIp"][2]
	    ip["ifStatic.IPByte3"] = ip["ifStatic.StaticIp"][3]
	    ip["ifStatic.IPByte4"] = ip["ifStatic.StaticIp"][4]
                    
        -- end: split ip address
	end
	
	if(ip["ifStatic.NetMask"] ~= nil and ip["ifStatic.NetMask"] ~= "") then
	    ip["ifStatic.NetMask"] = util.split(ip["ifStatic.NetMask"], ".")
	    
	    --begin: split subnet address
	    
	    ip["ifStatic.SMByte1"] = ip["ifStatic.NetMask"][1]
	    ip["ifStatic.SMByte2"] = ip["ifStatic.NetMask"][2]
	    ip["ifStatic.SMByte3"] = ip["ifStatic.NetMask"][3]
	    ip["ifStatic.SMByte4"] = ip["ifStatic.NetMask"][4]
                    
        -- end: split subnet address
	end
	
	vlanRow = util.tableAppend(vlanRow, ip)
	local net = db.getRow("networkInterface","LogicalIfName","LAN" .. vlanId) or {}
	vlanRow = util.tableAppend(vlanRow, net)
	local dhcp = db.getRow("dhcp","logicalIfName","LAN" .. vlanId) or {}
	if(dhcp["dhcp.startAddress"] ~= nil and dhcp["dhcp.startAddress"] ~= "") then
	    dhcp["dhcp.startAddress"] = util.split(dhcp["dhcp.startAddress"], ".")
	    
	    --begin: split start ip
	    
	    dhcp["dhcp.startAddress1"] = dhcp["dhcp.startAddress"][1]
	    dhcp["dhcp.startAddress2"] = dhcp["dhcp.startAddress"][2]
	    dhcp["dhcp.startAddress3"] = dhcp["dhcp.startAddress"][3]
	    dhcp["dhcp.startAddress4"] = dhcp["dhcp.startAddress"][4]
                    
        -- end: split start ip
	end
	
    if(dhcp["dhcp.endAddress"] ~= nil and dhcp["dhcp.endAddress"] ~= "") then
	    dhcp["dhcp.endAddress"] = util.split(dhcp["dhcp.endAddress"], ".")
	    
	    --begin: split end ip
	    
	    dhcp["dhcp.endAddress1"] = dhcp["dhcp.endAddress"][1]
	    dhcp["dhcp.endAddress2"] = dhcp["dhcp.endAddress"][2]
	    dhcp["dhcp.endAddress3"] = dhcp["dhcp.endAddress"][3]
	    dhcp["dhcp.endAddress4"] = dhcp["dhcp.endAddress"][4]
                    
        -- end: split end ip
	end
	
    if(dhcp["dhcp.primaryDnsServer"] ~= nil and dhcp["dhcp.primaryDnsServer"] ~= "") then
	    dhcp["dhcp.primaryDnsServer"] = util.split(dhcp["dhcp.primaryDnsServer"], ".")
	    
	    --begin: split primaryDnsServer ip
	    
	    dhcp["dhcp.primaryDnsServer1"] = dhcp["dhcp.primaryDnsServer"][1]
	    dhcp["dhcp.primaryDnsServer2"] = dhcp["dhcp.primaryDnsServer"][2]
	    dhcp["dhcp.primaryDnsServer3"] = dhcp["dhcp.primaryDnsServer"][3]
	    dhcp["dhcp.primaryDnsServer4"] = dhcp["dhcp.primaryDnsServer"][4]
                    
        -- end: split primaryDnsServer ip
	end
	
    if(dhcp["dhcp.secondaryDnsServer"] ~= nil and dhcp["dhcp.secondaryDnsServer"] ~= "") then
	    dhcp["dhcp.secondaryDnsServer"] = util.split(dhcp["dhcp.secondaryDnsServer"], ".")
	    
	    --begin: split secondaryDnsServer ip
	    
	    dhcp["dhcp.secondaryDnsServer1"] = dhcp["dhcp.secondaryDnsServer"][1]
	    dhcp["dhcp.secondaryDnsServer2"] = dhcp["dhcp.secondaryDnsServer"][2]
	    dhcp["dhcp.secondaryDnsServer3"] = dhcp["dhcp.secondaryDnsServer"][3]
	    dhcp["dhcp.secondaryDnsServer4"] = dhcp["dhcp.secondaryDnsServer"][4]
                    
        -- end: split secondaryDnsServer ip
	end
	
    if(dhcp["dhcp.winsServer"] ~= nil and dhcp["dhcp.winsServer"] ~= "") then
	    dhcp["dhcp.winsServer"] = util.split(dhcp["dhcp.winsServer"], ".")
	    
	    --begin: split winsServer ip
	    
	    dhcp["dhcp.winsServer1"] = dhcp["dhcp.winsServer"][1]
	    dhcp["dhcp.winsServer2"] = dhcp["dhcp.winsServer"][2]
	    dhcp["dhcp.winsServer3"] = dhcp["dhcp.winsServer"][3]
	    dhcp["dhcp.winsServer4"] = dhcp["dhcp.winsServer"][4]
                    
        -- end: split winsServer ip
	end
		
	local dhcpr = db.getRow("DhcpRelay","LogicalIfName","LAN" .. vlanId) or {}
	
	if(dhcpr["DhcpRelay.RelayGateway"] ~= nil and dhcpr["DhcpRelay.RelayGateway"] ~= "") then
	    dhcpr["dhcp.RelayGateway"] = util.split(dhcpr["DhcpRelay.RelayGateway"], ".")
	    
	    --begin: split winsServer ip
	    
	    dhcpr["dhcp.RelayGateway1"] = dhcpr["dhcp.RelayGateway"][1]
	    dhcpr["dhcp.RelayGateway2"] = dhcpr["dhcp.RelayGateway"][2]
	    dhcpr["dhcp.RelayGateway3"] = dhcpr["dhcp.RelayGateway"][3]
	    dhcpr["dhcp.RelayGateway4"] = dhcpr["dhcp.RelayGateway"][4]
                    
        -- end: split winsServer ip
	end
	
	-- Identify dhcpEnabled value
	
	if(dhcp["dhcp.dhcpEnabled"] == "0" and dhcpr["DhcpRelay.DhcpRelayStatus"] == "0") then
	    dhcp["dhcp.dhcpEnabled"] = "0"
	elseif(dhcp["dhcp.dhcpEnabled"] == "0" and dhcpr["DhcpRelay.DhcpRelayStatus"] == "1") then
	    dhcp["dhcp.dhcpEnabled"] = "1"
	elseif (dhcp["dhcp.dhcpEnabled"] == "1" and dhcpr["DhcpRelay.DhcpRelayStatus"] == "0") then
	    dhcp["dhcp.dhcpEnabled"] = "2"
	end
	vlanRow = util.tableAppend(vlanRow, dhcp)
	vlanRow = util.tableAppend(vlanRow, dhcpr)
	local interVlan = db.getRow("interVLANRouting","vlanId",vlanId1) or {}
	vlanRow = util.tableAppend(vlanRow, interVlan)
	local ldapRow = db.getRow("DhcpLdapServerInfo","LogicalIfName","LAN" .. vlanId) or {}
	vlanRow = util.tableAppend(vlanRow, ldapRow)	
	
	return vlanRow	
end

--[[
******************************************************************** 
* vlanConfig.addPortMemberShip - Used to get all the values
* check for which ports are checked for port membership
* if port is checked then check if entry for that port 
* exists in vlan table. If it exists then do not do any
* thing. If entry does not exist then add entry for that port 
* in vlan. Check for which port is unchecked. Then check if entry for
* that port exists in vlan. If it exists then delete that entry otherwise
* do not do anything*/

* Returns: true or false
--]]

function vlanConfig.addPortMemberShip (inputTable, vlanName, vlanId, portNum)
    DBTable = "vlan"
    local portName = "port" .. portNum
    local vlanKey = "vlan." .. portName
    local vlanRowId = db.existsRowWhere ("vlan", "vlanName='" .. vlanName .. "' and vlanId='" .. vlanId .."' and portName='" .. portName .. "'")
    if(inputTable[vlanKey] == "1") then               
        if(not vlanRowId) then
            -- Add a row in vlan table with portname as port4
            local portTbl = {}
            portTbl["vlan.vlanId"] = vlanId
            portTbl["vlan.vlanName"] = vlanName
            portTbl["vlan.portName"] = portName
            portTbl["vlan.vlanProfState"] = "1"
            valid = db.insert(DBTable, portTbl)
            if(not valid) then
                errorFlag, statusCode = "ERROR", "VLAN_CONFIG_FAILED"
            end
        end
    else                    
        if(vlanRowId) then
            valid = db.deleteRow ("vlan", "_ROWID_", vlanRowId)
            if(not valid) then
                  errorFlag, statusCode = "ERROR", "VLAN_CONFIG_FAILED"
            end
        end                
    end    
    if(errorFlag == "ERROR") then
        return false
    else
        return true
    end
end

--[[
***************************************************************************** 
* vlanConfig.vlanModify - wrapper function to call API to do enable or disable
* operation
* RETURNS: 'OK' or 'ERROR' 
*/
--]]

function vlanConfig.vlanModify (rows,operation)
		if (ACCESS_LEVEL ~= 0) then
		return "ACCESS_DENIED", "ADMIN_REQD"
	end

	db.beginTransaction() --begin transactions
    local valid = true
    local errStr = "STATUS_OK"
    local table = {}
	if(operation == "enable") then
		for k,v in pairs(rows) do
			local inputTable = {}
	    	local vlanId = v	    	
	    	local vlanMacOffsetType = db.getAttribute("vlanMac", "_ROWID_", "1","vlanMacOffsetType") or ''
		    local vlanUniqMacMaxCount = db.getAttribute("vlanMac", "_ROWID_", "1","vlanUniqueMacMaxCount") or 0
		    local vLanMacOfSet = db.getAttribute("vlanMac", "_ROWID_", "1","vlanLanMacOffset") or 0
		    vLanMacOfSet = tonumber(vLanMacOfSet) + 3
		    vlanUniqMacMaxCount = (vLanMacOfSet + tonumber(vlanUniqMacMaxCount)) - 1
	    	if(vlanMacOffsetType  == "0") then
	    		inputTable["vlanMacOffset"] = 0
	    	else
	    	    local countVlanMacOffSet = vlanConfig.vlanMacOffSetVal ()
	    	    if(countVlanMacOffSet ~= nil and (countVlanMacOffSet > vlanUniqMacMaxCount)) then
	    	        inputTable["vlanMacOffset"] = 0
	    	    else
	    	        inputTable["vlanMacOffset"] = countVlanMacOffSet
	    	    end
	    	end
			inputTable["vlanProfState"] = 1
			valid  = db.setAttributeWhere("vlan","vlanId='" .. vlanId .. "'","vlanMacOffset",inputTable["vlanMacOffset"])			
			valid  = valid and db.setAttributeWhere("vlan","vlanId='" .. vlanId .. "'","vlanProfState",inputTable["vlanProfState"])
			if(valid == false) then
				return "ERROR", "VLAN_ENABLE_FAILED"
			else
				local zoneTable = {}
				zoneTable["Zones.ZoneType"] = "SECURE"
				zoneTable["Zones.ZoneLogicalIfName"] = "LAN" .. v
				valid1 = db.insert("Zones", zoneTable) 
				whereQry= "LogicalIfName=LAN"..vlanId.." and AddressFamily=2"
			local mcRow = db.getRowWhere("ifStatic",whereQry)
			if (mcRow ~= nil) then
		     		local mcRowId = mcRow["ifStatic._ROWID_"]
				valid1 = valid1 and db.setAttribute("ifStatic","_ROWID_",mcRowId,"AddressFamily","2")
			else
			valid1 = valid1 and db.setAttribute("ifStatic","LogicalIfName","LAN"..vlanId,"AddressFamily","2")
			end
				 
				local oldStateEnbVal = db.getAttribute("dhcp","logicalIfName","LAN"..vlanId,"oldState")
				valid1 = valid1 and db.setAttribute("dhcp","logicalIfName","LAN"..vlanId,"dhcpEnabled",oldStateEnbVal)
				valid1 = valid1 and db.setAttribute("dhcp","logicalIfName","LAN"..vlanId,"oldState","0")		
				--valid1 = valid1 and db.setAttribute("tcpdump","logicalIfName","LAN"..vlanId,"tcpdumpEnabled","1")
				local existVal = db.getAttribute("vlanPortMgmt","oldVlanId",vlanId,"oldVlanId")
				if(existVal ~= nil) then
				    valid1 = valid1 and db.setAttribute("vlanPortMgmt","oldVlanId",vlanId,"DefaultPVID",existVal)
                    valid1 = valid1 and db.setAttribute("vlanPortMgmt","oldVlanId",vlanId,"oldVlanId","1")			    
				end
			end				
		end
	else
    -- Loop through the vlanIds received from the frontend                
    	for k,v in pairs(rows) do    		
	    	local inputTable = {}
	    	local vlanId = v	    	
			inputTable["vlanMacOffset"] = 0
			inputTable["vlanProfState"] = 0
			valid  = db.setAttributeWhere("vlan","vlanId='" .. vlanId .. "'","vlanMacOffset",inputTable["vlanMacOffset"])			
			valid  = valid and db.setAttributeWhere("vlan","vlanId='" .. vlanId .. "'","vlanProfState",inputTable["vlanProfState"])
				if(valid == false) then					
					return "ERROR", "VLAN_DISABLE_FAILED"
				else
					local oldStateVal = db.getAttribute("dhcp","logicalIfName","LAN"..vlanId,"dhcpEnabled")
					valid1 = valid and db.setAttribute("dhcp","logicalIfName","LAN"..vlanId,"oldState",oldStateVal)
					valid1 = valid1 and db.setAttribute("dhcp","logicalIfName","LAN"..vlanId,"dhcpEnabled","0")			
					valid1 = valid1 and db.setAttribute("tcpdump","logicalIfName","LAN"..vlanId,"tcpdumpEnabled","0")
                    local oldDefPVID = db.getAttribute("vlanPortMgmt","DefaultPVID",vlanId,"DefaultPVID")
                    if(oldDefPVID ~= nil) then
                        valid1 = valid1 and db.setAttribute("vlanPortMgmt","DefaultPVID",vlanId,"oldVlanId",oldDefPVID)
					    valid1 = valid1 and db.setAttribute("vlanPortMgmt","DefaultPVID",vlanId,"DefaultPVID","1")
                    end			
					valid1 = valid1 and db.deleteRow("Zones","ZoneLogicalIfName","LAN"..vlanId)					
				end
        end  
     end      
        
        
        if(valid) then
        	db.commitTransaction(true)
            if(operation == "enable") then
                os.execute ("/pfrm2.0/bin/snmpTrapConfig 42 Major Custom VLAN Enabled")
            else
                os.execute ("/pfrm2.0/bin/snmpTrapConfig 42 Major Custom VLAN Disabled")
            end
            return "OK", "STATUS_OK"
        else
	    db.rollback()
            return "ERROR", "VLAN_CONFIG_IN_USE"
        end
end

--[[
***************************************************************************** 
* vlanConfig.vlanDefVlan - wrapper function to edit Default vlan in port management
* operation
* RETURNS: 'OK' or 'ERROR' 
*/
--]]

function vlanConfig.vlanDefVlan (inputTable)
	if (ACCESS_LEVEL ~= 0) then
	  return "ACCESS_DENIED", "ADMIN_REQD"
	end

	db.beginTransaction() --begin transactions
    local valid = true
    local errStr = "STATUS_OK"         
   
    valid = db.setAttribute("VlanPortMgmt","PortName","Port1","DefaultPVID",inputTable["VlanPortMgmt.DefPort1"])	
    valid = valid and db.setAttribute("VlanPortMgmt","PortName","Port2","DefaultPVID",inputTable["VlanPortMgmt.DefPort2"])	
    valid = valid and db.setAttribute("VlanPortMgmt","PortName","Port3","DefaultPVID",inputTable["VlanPortMgmt.DefPort3"])	    
   	if(inputTable["VlanPortMgmt.DefPort4"] ~= nil) then
   	   valid = valid and db.setAttribute("VlanPortMgmt","PortName","Port4","DefaultPVID",inputTable["VlanPortMgmt.DefPort4"])    end
     
     if(valid) then
     		db.commitTransaction(true)
            return "OK", "STATUS_OK"
        else
	    db.rollback ()
            return "ERROR", "VLAN_DEF_CONFIG_FAILED"
        end
end

--[[
***************************************************************************** 
* vlanConfig.vlanMacConfig - wrapper function to edit Default vlan in port management
* operation
* RETURNS: 'OK' or 'ERROR' 
*/
--]]

function vlanConfig.vlanMacConfig (inputTable, rowid, operation)
	-- if not allowed to edit
	if (ACCESS_LEVEL ~= 0) then
		return "ACCESS_DENIED", "ADMIN_REQD"
	end
	db.beginTransaction() --begin transaction
	local valid = false	
   	valid = db.update("vlanMac", inputTable, rowid)
   	
   	local vlanUniMacMaxVal = db.getAttribute("vlanMac", "_ROWID_", "1","vlanUniqueMacMaxCount") or 0
    vlanUniMacMaxVal = tonumber(vlanUniMacMaxVal)
    local vlanLanMacOfset = db.getAttribute("vlanMac", "_ROWID_", "1","vlanLanMacOffset") or 0
    vlanLanMacOfset = tonumber(vlanLanMacOfset) + 1
    vlanUniMacMaxVal = (vlanLanMacOfset + vlanUniMacMaxVal) - 1
    
    local tmpvlanTbl = db.getRowsWhere("vlan","vlanName != 'Default' and vlanId!=4094 and portName = 'port10'")
    
   	local macValue = vlanLanMacOfset
   	local countVlanMac   	
   	if(valid) then
   	    if(tonumber(inputTable["vlanMac.vlanMacOffsetType"]) == 0) then
            if (db.existsRowWhere("vlan","vlanName!='Default' and portName='port10' and vlanId!=4094")) then
   	            valid = db.setAttributeWhere("vlan","vlanName!='Default' and portName='port10' and vlanId!=4094","vlanMacOffset","0")
            end
   	    else
   	        for countVlanMac = vlanLanMacOfset,vlanUniMacMaxVal do
   	            if(db.existsRowWhere("vlan", "vlanProfState = '1' and vlanMacOffset='" .. countVlanMac .. "'")) then
   	                macValue = macValue + 1
   	            end
   	        end	        
	        for k, v in pairs (tmpvlanTbl) do            
               if(macValue > vlanUniMacMaxVal) then
                   valid = db.setAttributeWhere("vlan","_ROWID_ = " .. v["vlan._ROWID_"],"vlanMacOffset","0")
               else
                   valid = db.setAttributeWhere("vlan","_ROWID_ = " .. v["vlan._ROWID_"],"vlanMacOffset",macValue)
                   macValue = macValue + 1
               end
	        end
	    end
	end
    if (valid) then
        --dummy update on ifStatic Table to assign ip address to VLAN
        db.setAttributeWhere ("ifStatic", "LogicalIfName LIKE 'LAN%' and LogicalIfName!='LAN'", "OID", "0")
    end

	-- return
	if (valid) then
		db.commitTransaction(true)
	    return "OK", "STATUS_OK"            
	else
		db.rollback()
		return "ERROR", "VMAC_VLANMAC_CONFIG_FAILED"
	end
end


function vlanConfig.vlanMacOffSetVal ()    
    local vlanMacMaxCount = db.getAttribute("vlanMac", "_ROWID_", "1","vlanUniqueMacMaxCount") or 0
    vlanMacMaxCount = tonumber(vlanMacMaxCount)
    local vlanLanMacOffset = db.getAttribute("vlanMac", "_ROWID_", "1","vlanLanMacOffset") or 0
    vlanLanMacOffset = tonumber(vlanLanMacOffset) + 3
    local macOffsetVal = vlanLanMacOffset
    vlanMacMaxCount = (vlanLanMacOffset + vlanMacMaxCount) - 1
    local countVlanMac
    for countVlanMac = vlanLanMacOffset,vlanMacMaxCount do
        if(db.existsRowWhere("vlan", "vlanProfState = '1' and vlanMacOffset='" .. countVlanMac .. "'")) then
            macOffsetVal = macOffsetVal + 1
        else
            break
        end
    end
    return macOffsetVal
end

