--[[
#### Vivek Dutt Sharma.
#### TeamF1
#### www.TeamF1.com
#### Nov 3, 2009
#### File: wanPort.lua
#### Description: wanPort functions
#### Revisions:
#### Revisions:01b, 23oct12,cg changes to configure IPv6 advanced options
]]--

--************* Initial Code *************
--package wan
wan = {}

--************* Functions *************
--[[
/******************************************************************************
*
* wan.config - hadles all data base manipulation operations.
*
* RETURNS: true or false
*/
]]--
function wan.config (inputTable, rowid, operation)
    -- validate
    if (db.typeAndRangeValidate(inputTable)) then
        if (operation == "add") then
            return db.insert(DBTable, inputTable)
        elseif (operation == "edit") then
            return db.update(DBTable, inputTable, rowid)
        elseif (operation == "delete") then
            return false
        end
    else
        return false
    end
end

--[[
/******************************************************************************
*
* wan.portConfig - updates WAN port configurations
*
* This routine updates data base for the following configurations.
* 1. Set Port Speed
* 2. Set MAC Address Option
* 
* RETURNS: OK or ERROR
*/
]]--
function wan.portConfig (inputTable, logicalName)
    local valid = false
    
    -- Check for multicast mac address
    --
    if(inputTable["Wan.UseMac"] == "2" ) then
        local macAddr = inputTable["Wan.MacAddr"]
        local secondBit = string.sub(macAddr,2,2)
        if(secondBit == "1" or secondBit == "3" or secondBit == "5" or secondBit == "7" or secondBit == "9" or secondBit == "b" or secondBit == "d" or secondBit == "f" or secondBit == "B" or secondBit == "D" or secondBit == "F") then
            return "ERROR", "MULTICAST_MAC_ERROR"
        end
    end
    -- 
    --
    local PortMgmtRow = nil
    PortMgmtRow = db.getRowWhere ("PortMgmt","PortName= '" .. logicalName .. "'") 
    
    --Port Management
    if(inputTable["Wan.portSpeed"] == "0") then
    	PortMgmtRow["PortMgmt.AutoNegEnable"] = "1"
    else	
    	PortMgmtRow["PortMgmt.AutoNegEnable"] = "0"
    end
    if(inputTable["Wan.portSpeed"] == "1" or inputTable["Wan.portSpeed"] == "3" or inputTable["Wan.portSpeed"] == "5" ) then
    	PortMgmtRow["PortMgmt.DuplexMode"]= "0"
    end	
    if(inputTable["Wan.portSpeed"] == "2" or inputTable["Wan.portSpeed"] == "4" or inputTable["Wan.portSpeed"] == "6" ) then
    	PortMgmtRow["PortMgmt.DuplexMode"] = "1"
    end    
    if(inputTable["Wan.portSpeed"] == "1" or inputTable["Wan.portSpeed"] == "2") then
    	PortMgmtRow["PortMgmt.PortSpeed"]= "10"
    end
    if(inputTable["Wan.portSpeed"]=="3" or inputTable["Wan.portSpeed"]=="4") then
    	PortMgmtRow["PortMgmt.PortSpeed"]= "100"
    end
    if(inputTable["Wan.portSpeed"]=="5" or inputTable["Wan.portSpeed"]=="6") then
    	PortMgmtRow["PortMgmt.PortSpeed"]= "1000"
    end

    DBTable = "PortMgmt"
    valid = wan.config (PortMgmtRow,PortMgmtRow["PortMgmt._ROWID_"], "edit")
    if (not valid) then		
	return "ERROR", "PORT_MGMT_CONFIG_FAILED"
    end

    -- getting NimfConf table rows
    local NimfTable = db.getRowsWhere ("NimfConf","LogicalIfName = '" .. logicalName.. "' and AddressFamily = 2")       
    
    if (NimfTable == nil) then
        return "ERROR", "NIMF_CONFIG_NOTFOUND"
    end 
    
    -- config NIMF
    -- Set the wan port configuration
    local i=0 
    for k,v in pairs(NimfTable) do
        i = i + 1
        local NimfRow = NimfTable[i]
        NimfRow["NimfConf.UseDefaultMtu"] = inputTable["Wan.DefMtu"]
        NimfRow["NimfConf.SetMacAddress"] = inputTable["Wan.UseMac"]              
        if (inputTable["Wan.DefMtu"] == "0") then
            NimfRow["NimfConf.MtuSize"] = inputTable["Wan.MtuSize"]
        else
            if (NimfRow["NimfConf.RequireLogin"] == "0") then
                NimfRow["NimfConf.MtuSize"] = "1500"
            else
                NimfRow["NimfConf.MtuSize"] = "1492"
            end
        end 
       if (inputTable["Wan.UseMac"] == "2") then
           NimfRow["NimfConf.MacAddress"] = inputTable["Wan.MacAddr"]
       end            
        DBTable = "NimfConf"
        valid = wan.config (NimfRow ,NimfRow["NimfConf._ROWID_"], "edit")

        if (not valid ) then
            return "ERROR", "NIMF_CONFIG_NOTFOUND"
        end     
    end

   local failureTable = {}
    failureTable["WanFailure.LogicalIfName"]=logicalName
    failureTable["WanFailure.AddressFamily"]="2"
    failureTable["WanFailure.FailureDetectionMeth"] = inputTable["Wan.FailureDetectionMeth"]
    failureTable["WanFailure.PingIp"]=inputTable["Wan.PingIp"]
    failureTable["WanFailure.DnsIp"]=inputTable["Wan.DnsIp"]
    failureTable["WanFailure.RetryTime"]=inputTable["Wan.RetryTime"]
    failureTable["WanFailure.RetryAttempts"]=inputTable["Wan.RetryAttempts"]

    if(valid) then
	    if(db.existsRow("WanFailure","LogicalIfName",logicalName)) then
            local rowId = db.getAttribute("WanFailure","LogicalIfName",logicalName,"_ROWID_")
	        valid=db.update("WanFailure",failureTable,rowId)
    	else
	        valid=db.insert("WanFailure",failureTable)
    	end
    end    
    
 if (not valid ) then
            return "ERROR", "WF_CONFIG_NOTFOUND"
  end

 local speedTable = {}
    speedTable["WanSpeed.LogicalIfName"]=logicalName
    speedTable["WanSpeed.AddressFamily"]="2"    
    speedTable["WanSpeed.WanConnectionType"]=inputTable["Wan.WanConnectionType"]
    speedTable["WanSpeed.UploadSpeedType"]=inputTable["Wan.UploadSpeedType"]
    if (speedTable["WanSpeed.UploadSpeedType"]== "0") then
        speedTable["WanSpeed.UploadSpeed"]=inputTable["Wan.UploadSpeed"]
    else
         speedTable["WanSpeed.UploadSpeed"]=inputTable["Wan.UploadSpeedType"]
    end
    speedTable["WanSpeed.DownloadSpeedType"]=inputTable["Wan.DownloadSpeedType"]
    if (speedTable["WanSpeed.DownloadSpeedType"]== "0") then
        speedTable["WanSpeed.DownloadSpeed"]=inputTable["Wan.DownloadSpeed"]
    else
         speedTable["WanSpeed.DownloadSpeed"]=inputTable["Wan.DownloadSpeedType"]
    end    

    if(valid) then
	    if(db.existsRow("WanSpeed","LogicalIfName",logicalName)) then
            local rowId = db.getAttribute("WanSpeed","LogicalIfName",logicalName,"_ROWID_")
	        valid=db.update("WanSpeed",speedTable,rowId)
    	else
	        valid=db.insert("WanSpeed",speedTable)
    	end
    end    
    
 if (not valid ) then
            return "ERROR", "WS_CONFIG_NOTFOUND"
        end

    return "OK", "STATUS_OK"
end

--[[
/******************************************************************************
*
* wan.wanConfigSetWrap - Wrapper function to set different options of WAN1 and WAN2
*
* This routine updates data base for the following configurations.
* 1. Checks for Multicast MAC Address
* 2. Sets various wan port options.
* 
* RETURNS: ACCESS_DENIED or OK or ERROR
*/
]]--
function wan.wanConfigSetWrap (inputTable)
	-- if not allowed to edit
	if (ACCESS_LEVEL ~= 0) then
		return "ACCESS_DENIED", "ADMIN_REQD"
	end
    local logIfName = inputTable["Wan.wanName"]
    util.appendDebugOut("Logical If Name.."..logIfName)
    

    db.beginTransaction() --begin transaction
    local valid = false
    
    -- Configure WAN1 Port Settings           
        local errFg, stCode = wan.portConfig (inputTable, logIfName)
        if (errFg ~= "OK") then
		db.rollback()
    		return errFg, stCode
        else
            valid = true
        end
    -- return "OK", if succeed all updates
    -- or
    -- return "ERROR", if update failed and also Roolback the database.
	if (valid) then
		db.commitTransaction(true)
		return "OK", "STATUS_OK"
	else
		db.rollback()
		return "ERROR", "NIMF_CONFIG_FAILED"
	end
end

--[[
/******************************************************************************
* wan.wanConfigGetWrap - Wrapper function to get WAN Configuration
*
* RETURNS: A lua table contains WAN configuration
*/
]]--
function wan.wanConfigGetWrap (wanLogicalIfName)
    local logIfName = wanLogicalIfName   
    local outTable= wan.wanAdvConfigGet (logIfName )   
    return outTable
end

--[[
/******************************************************************************
* wan.wanAdvConfigGet - Wrapper function to get WAN Advanced Configurations
*
* RETURNS: A lua table contains WAN Advanced Configurations
*/
]]--
function wan.wanAdvConfigGet (logIfName)
    local outTable = {}
    local PortMgmtRow 

    PortMgmtRow = db.getRowWhere ("PortMgmt","PortName= '" .. logIfName .. "'")

    local NimfRow = db.getRowWhere ("NimfConf","LogicalIfName = '" .. logIfName .. "' and AddressFamily = 2")
    if (NimfRow ~= nil) then
        outTable["Wan.DefMtu"] = NimfRow["NimfConf.UseDefaultMtu"] or ''
        outTable["Wan.MtuSize"] = NimfRow["NimfConf.MtuSize"] or ''
        outTable["Wan.UseMac"] = NimfRow["NimfConf.SetMacAddress"] or ''
        outTable["Wan.MacAddr"] = NimfRow["NimfConf.MacAddress"] or ''
        if (NimfRow["NimfConf.RequireLogin"] == "0") then
            outTable["Wan.MtuSizeMin"] = "68"
            outTable["Wan.MtuSizeMax"] = "1500"
        else
            outTable["Wan.MtuSizeMin"] = "128"
            outTable["Wan.MtuSizeMax"] = "1492"
        end        
    end
    if ( PortMgmtRow["PortMgmt.AutoNegEnable"] == "0" ) then
        if (PortMgmtRow["PortMgmt.PortSpeed"] == "10" and  PortMgmtRow["PortMgmt.DuplexMode"] == "0") then
            outTable["Wan.portSpeed"] = "1"
        end
        if (PortMgmtRow["PortMgmt.PortSpeed"] == "10" and  PortMgmtRow["PortMgmt.DuplexMode"] == "1") then
            outTable["Wan.portSpeed"] = "2"
        end
        if (PortMgmtRow["PortMgmt.PortSpeed"] == "100" and  PortMgmtRow["PortMgmt.DuplexMode"] == "0") then
            outTable["Wan.portSpeed"] = "3"
        end
        if (PortMgmtRow["PortMgmt.PortSpeed"] == "100" and  PortMgmtRow["PortMgmt.DuplexMode"] == "1") then
            outTable["Wan.portSpeed"] = "4"
        end
        if (PortMgmtRow["PortMgmt.PortSpeed"] == "1000" and  PortMgmtRow["PortMgmt.DuplexMode"] == "0") then
            outTable["Wan.portSpeed"] = "5"
        end
        if (PortMgmtRow["PortMgmt.PortSpeed"] == "1000" and  PortMgmtRow["PortMgmt.DuplexMode"] == "1") then
            outTable["Wan.portSpeed"] = "6"
        end
    else
        outTable["Wan.portSpeed"] = "0"
    end


    local WanFailureConf = db.getRowWhere ("WanFailure","LogicalIfName = '" .. logIfName .. "' and AddressFamily = 2")
    if(WanFailureConf ~= nil) then
    outTable["Wan.FailureDetectionMeth"] = WanFailureConf["WanFailure.FailureDetectionMeth"]
    outTable["Wan.DnsIp"] = WanFailureConf["WanFailure.DnsIp"]
    outTable["Wan.PingIp"] = WanFailureConf["WanFailure.PingIp"]
    outTable["Wan.RetryTime"] = WanFailureConf["WanFailure.RetryTime"]
    outTable["Wan.RetryAttempts"] = WanFailureConf["WanFailure.RetryAttempts"]
    end

local WanSpeedConf = db.getRowWhere ("WanSpeed","LogicalIfName = '" .. logIfName .. "'")
if(WanSpeedConf ~= nil) then             
    outTable["Wan.WanConnectionType"] = WanSpeedConf["WanSpeed.WanConnectionType"]
    outTable["Wan.UploadSpeedType"] = WanSpeedConf["WanSpeed.UploadSpeedType"]
    outTable["Wan.UploadSpeed"] = WanSpeedConf["WanSpeed.UploadSpeed"]
    outTable["Wan.DownloadSpeedType"] = WanSpeedConf["WanSpeed.DownloadSpeedType"]
    outTable["Wan.DownloadSpeed"] = WanSpeedConf["WanSpeed.DownloadSpeed"]
    end
	return outTable
end

--[[
/******************************************************************************
*
* wan.wanConfigSetWrap - Wrapper function to set different options of WAN1 and WAN2
*
* This routine updates data base for the following configurations.
* 1. Checks for Multicast MAC Address
* 2. Sets various wan port options.
* 
* RETURNS: ACCESS_DENIED or OK or ERROR
*/
]]--
function wan.wan6AdvConfigSet (inputTable)
	-- if not allowed to edit
	if (ACCESS_LEVEL ~= 0) then
		return "ACCESS_DENIED", "ADMIN_REQD"
	end
    local wanFailureRow
    local valid = false
    local wanName = inputTable["WanFailure.LogicalIfName"]
    util.appendDebugOut("Logical If Name.."..wanName)
    
    wanFailureRow = db.getRowWhere ("WanFailure", "LogicalIfName = '" .. wanName .. "' AND AddressFamily = 10")
    
    wanFailureRow["WanFailure.FailureDetectionMeth"] = "3"
    wanFailureRow["WanFailure.PingIp"] = inputTable ["WanFailure.PingIp"]
    wanFailureRow["WanFailure.RetryTime"] = inputTable ["WanFailure.RetryTime"] 
    wanFailureRow["WanFailure.RetryAttempts"] = inputTable ["WanFailure.RetryAttempts"] 
    
    valid = db.update ("WanFailure", wanFailureRow, wanFailureRow["WanFailure._ROWID_"])
	if (valid) then
		return "OK", "STATUS_OK"
	else
		return "ERROR", "NIMF_CONFIG_FAILED"
	end
end

