--[[
#### Chaitanya Dandamudi
#### TeamF1
#### www.TeamF1.com
#### December 12, 2007

#### File: x509.lua
#### Description: x509 functions

#### Revisions:
01h, 11jan13,sig fixed for SPR 36427
01g, 04sep12,su  introduced crlLoad for CRL upload.
01f, 12jul12,sig Fixed SPR 33513
01e, 1feb12,  added multiple csr support 
01d, 22mar10, rac changed to capture extra error messages while adding active 
              self certificates(part of 17062 spr fix)
01c, 15nov08, til changed signature of certLoad for identifying the input
              certificate type(ca/self).   
01b, 22oct08, til included support for httpdMgmt tbl update
              on deletion of active certificate.
]]--


--************* Requires *************

require "x509Lib"

--************* Initial Code *************

--package x509
x509 = {}

--table packages
cert = {}

--************* Functions *************

-- x509 inputValidate
function x509.inputValidate (inputTable, operation)
      --[[  if (true) then
            return db.typeAndRangeValidate(inputTable)
        end
        return false ]]--
end
        
-- x509 configuration
function x509.uploadTrustedCACert (inputTable)
    -- if not allowed to edit
        if (ACCESS_LEVEL ~= 0) then
                NextPage = "certificates"
                return "ACCESS_DENIED", "ADMIN_REQD"
        end
        db.beginTransaction() --begin transaction
        local valid = false

        -- uploading Certificate
       	util.appendDebugOut(util.tableToStringRec(inputTable))
    	local fileSize = inputTable["file.upload"]["filesize"]
	local fileName = inputTable["file.upload"]["filename"]
    	local fileHandle =  inputTable["file.upload"]["file"]
	-- moving file to required path
	local fileName = cgilua.cookies.get("TeamF1Login")
	local valid = os.execute("mv" .. " " .. "/tmp/" .. 
                             fileName .. " /tmp/" .. "temp.crt")

    local certInputTable = {}
    local certData
    certData = util.fileToString("/tmp/temp.crt")
    
    local status,name,subjectName,issuerName,expiryTime,serialNumber,format =
                                                    x509Lib.certLoad(certData, "ca")
                                                    
    local valid = os.execute("rm" .. " /tmp/" .. "temp.crt")

    if (status == -1) then
        return "ERROR", "X509_INVALID_CA"
    end 
 
    certInputTable["cert.name"] = name
    certInputTable["cert.subjectName"] = subjectName
    certInputTable["cert.issuerName"] = issuerName
    certInputTable["cert.expiryTime"] = expiryTime
    certInputTable["cert.serialNumber"] = serialNumber
    certInputTable["cert.format"] = format
    certInputTable["cert.certType"] = "ca"
    certInputTable["cert.certData"] = certData
    valid = db.insert("cert",certInputTable)
	
    -- return
	if (valid) then
		db.commitTransaction(true)		
		return "OK", "X509_CA_ADD_SUCCESS"
	else
		db.rollback()
		return "ERROR", "X509_CA_ADD_FAILURE"
	end
end

--Trusted Cert Delete
function x509.trustedCACertDelete (rowIds)

	if (ACCESS_LEVEL ~= 0) then
		return "ACCESS_DENIED", "ADMIN_REQD"
	end

	db.beginTransaction() --begin transactions

	-- delete Trusted Certificates
        for k,v in pairs(rowIds) do
            local inTable = {}
            inTable["cert._ROWID_"] = v;

			-- Do not delete a CA certificate if there is another certificate signed with it
			local subjectName = db.getAttribute("cert", "cert._ROWID_", v, "subjectName")
			local whereClause = "issuerName = '" .. subjectName .. "' AND certType = 'self'"
			local rows = db.getRowsWhere("cert", whereClause)
			if (rows and #rows >= 1) then
				return "ERROR", "X509_CA_BEING_USED_DEL_FAILED"
			end
          
            -- delete <hash_of_certificate>.0 file
           local status,subjectHash = x509Lib.getCertHash(v)
          
           if (status == -1) then
               return "ERROR", "X509_CA_DEL_FAILURE"
           end     
           
           os.execute("rm" .. " /var/certs/ca/" .. subjectHash .. ".0" )
          
           -- delete CA<rowId>.crt file 
           os.execute("rm" .. " /var/certs/ca/" .. "CA" .. v .. ".crt" )
         
           valid = db.delete("cert", inTable)
	end

	-- return
	if (valid) then
		db.commitTransaction(true)
		return "OK", "X509_CA_DEL_SUCCESS"
	else
		db.rollback()
		return "ERROR", "X509_CA_DEL_FAILURE"
	end

end

--CRL Certificate Delete
function x509.crlCertDelete (rowIds)

	if (ACCESS_LEVEL ~= 0) then
		return "ACCESS_DENIED", "ADMIN_REQD"
	end

	db.beginTransaction() --begin transactions

	-- delete Trusted Certificates
        for k,v in pairs(rowIds) do
            local inTable = {}
            inTable["x509Crl._ROWID_"] = v;
		    valid = db.delete("x509Crl", inTable)
     	end

	if (valid) then       
        -- commit the transaction
	    db.commitTransaction(true)
	    -- return
		return "OK", "X509_CRL_DEL_SUCCESS"
	else
		db.rollback()
		return "ERROR", "X509_CRL_DEL_FAILURE"
	end

end

function x509.uploadCRLCert (inputTable)
    -- if not allowed to edit
        if (ACCESS_LEVEL ~= 0) then
                NextPage = "certificates"
                return "ACCESS_DENIED", "ADMIN_REQD"
        end
        db.beginTransaction() --begin transaction
        local valid = false

        -- uploading Certificate
       	util.appendDebugOut(util.tableToStringRec(inputTable))
    	local fileSize = inputTable["file.upload"]["filesize"]
	local fileName = inputTable["file.upload"]["filename"]
    local checkFileType = util.split(fileName,".")
    if (checkFileType[2] ~= "pem") then
        return "ERROR","Failed to Upload crl file. Please upload .pem extension file"
    end

    	local fileHandle =  inputTable["file.upload"]["file"]
	-- moving file to required path
	local fileName = cgilua.cookies.get("TeamF1Login")
	local valid = os.execute("mv" .. " " .. "/tmp/" .. 
                             fileName .. " /tmp/" .. "temp.crt")

    local certInputTable = {}
    local certData
    certData = util.fileToString("/tmp/temp.crt")
    
    local status,crlFileName,crlSubjectName,crlIdName,crlLUpdate,crlLNUpdate =
                                                    x509Lib.crlLoad("/tmp/temp.crt", "crl")
        
    local valid = os.execute("rm" .. " /tmp/" .. "temp.crt")

    if (status == -1) then
        return "ERROR", "X509_INVALID_CRL"
    end 
 
    certInputTable["x509Crl.name"] = crlFileName
    certInputTable["x509Crl.subjectName"] = crlSubjectName
    certInputTable["x509Crl.crlIdName"] = crlIdName
    certInputTable["x509Crl.crlLUpdate"] = crlLUpdate
    certInputTable["x509Crl.crlLNUpdate"] = crlLNUpdate
    certInputTable["x509Crl.certData"] = certData
    certInputTable["x509Crl.certType"] = "crl"   
    valid = db.insert("x509Crl",certInputTable)
	
    -- return
	if (valid) then
		db.commitTransaction(true)		
		return "OK", "X509_CRL_ADD_SUCCESS"
	else
		db.rollback()
		return "ERROR", "X509_CRL_ADD_FAILURE"
	end
end

--Active Self Cert Delete
function x509.activeSelfCertDelete (rowIds)
      
	if (ACCESS_LEVEL ~= 0) then
		return "ACCESS_DENIED", "ADMIN_REQD"
	end

	db.beginTransaction() --begin transactions

	-- delete Trusted Certificates
        for k,v in pairs(rowIds) do
            local inTable = {}
            inTable["cert._ROWID_"] = v;
		    valid = db.delete("cert", inTable)
     	end

	if (valid) then
        -- Restart httpd
        -- get the column 'certEvent''s value from httpsMgmt
        httpsMgmtRow = db.getRow ("httpsMgmt","_ROWID_","1")
        -- toggle the value of 'certEvent'
        if (httpsMgmtRow) then 
            if (httpsMgmtRow["httpsMgmt.certEvent"] == "1") then
                httpsMgmtRow["httpsMgmt.certEvent"] = "0"
            elseif (httpsMgmtRow["httpsMgmt.certEvent"] == "0") then
                httpsMgmtRow["httpsMgmt.certEvent"] = "1"
            end -- certEventFlag    
            -- update the table 'httpsMgmt' with new 'certEvent' value.
            valid = db.update("httpsMgmt", httpsMgmtRow, "1")
        end --httpsMgmtRow
        
        -- commit the transaction
	    db.commitTransaction(true)
	    -- return
		return "OK", "X509_SELF_DEL_SUCCESS"
	else
		db.rollback()
		return "ERROR", "X509_SELF_DEL_FAILURE"
	end

end


-- Generate Self Certificate
function x509.genSelfCert (inputTable)
    -- if not allowed to edit
        if (ACCESS_LEVEL ~= 0) then
                NextPage = "certificates"
                return "ACCESS_DENIED", "ADMIN_REQD"
        end

        db.beginTransaction() --begin transaction
        local valid = false
        -- append CN= for subjectName if '=' is not found in the input string 
        local stIdx,endIdx = string.find(inputTable["x509SelfCertReq.subjectName"],"=")
        if (stIdx == nil and endIdx == nil) then
            inputTable["x509SelfCertReq.subjectName"] = "CN=" .. inputTable["x509SelfCertReq.subjectName"]
        end
        
        --check for duplicate entry
        if (db.existsRow("x509SelfCertReq", "requestName", inputTable["x509SelfCertReq.requestName"])) then
             db.rollback()
             return "ERROR", "X509_SELF_SAME_REQUESTNAME"
        end
        
        --add the new entry
        valid = db.insert("x509SelfCertReq", inputTable)
    
     -- return
	if (valid) then
		db.commitTransaction(true)
		return "OK", "STATUS_OK"
	else
		db.rollback()
		return "ERROR", "X509_SELF_GEN_FAILURE"
	end
end
 

--Self Cert Delete
function x509.selfCertDelete (rowIds)

	if (ACCESS_LEVEL ~= 0) then
		return "ACCESS_DENIED", "ADMIN_REQD"
	end

	db.beginTransaction() --begin transactions

	-- delete Self Certificates
        for k,v in pairs(rowIds) do
            local inTable = {}
        	inTable["x509SelfCertReq._ROWID_"] = v;	
            local requestStatus = db.getAttribute("x509SelfCertReq", "x509SelfCertReq._ROWID_", v, "requestStatus")
            local appType = db.getAttribute("x509SelfCertReq", "x509SelfCertReq._ROWID_", v, "appType")
			--check to prevent used csr from deletion
            if (requestStatus == "1") then
				db.rollback()		
				return "ERROR", "X509_SELF_BEING_USED"
        	end			
		    valid = db.delete("x509SelfCertReq", inTable)
  		    -- return
			if (valid) then
				os.execute("rm -f /var/certs/self/" .. appType .. v .. ".csr")		
				db.commitTransaction(true)
			else
				db.rollback()
			end
		end

    -- return
	if (valid) then
		return "OK", "X509_SELF_DEL_SUCCESS"
	else
		return "ERROR", "X509_SELF_DEL_FAILURE"
	end

end


-- upload Active Self Certificates
function x509.uploadSelfCert (inputTable)
    -- if not allowed to edit
        if (ACCESS_LEVEL ~= 0) then
                NextPage = "certificates"
                return "ACCESS_DENIED", "ADMIN_REQD"
        end
        db.beginTransaction() --begin transaction
        local valid = false
        local checkSelfCert = db.getRow("cert","certType","self")
        if (checkSelfCert) then
            return "ERROR", "MULIPLE_UPLOAD_ERROR"
        end
        -- uploading Certificate
       	util.appendDebugOut(util.tableToStringRec(inputTable))
    	local fileSize = inputTable["file.upload"]["filesize"]
	local fileName = inputTable["file.upload"]["filename"]
    	local fileHandle =  inputTable["file.upload"]["file"]
	-- moving file to required path
	local fileName = cgilua.cookies.get("TeamF1Login")
	local valid = os.execute("mv" .. " " .. "/tmp/" .. 
                             fileName .. " /tmp/" .. "temp.crt")

    local certInputTable = {}
    local certData
    certData = util.fileToString("/tmp/temp.crt")
    
    local status,name,subjectName,issuerName,expiryTime,serialNumber,format =
                                                    x509Lib.certLoad(certData, "self")
                                                    
    local valid = os.execute("rm" .. " /tmp/" .. "temp.crt")

    if (status == -1) then
        return "ERROR", "X509_INVALID_SELF"
    elseif (status == 27) then        
        return "ERROR", "X509_INVALID_SELF_CA"
    elseif (status == 26) then        
        return "ERROR", "X509_INVALID_SELF_PURPOSE"        
    end   
    --copy the requestname of used csr to the name attribute of uploaded cert 
    certInputTable["cert.name"] = name
    certInputTable["cert.subjectName"] = subjectName
    certInputTable["cert.issuerName"] = issuerName
    certInputTable["cert.expiryTime"] = expiryTime
    certInputTable["cert.serialNumber"] = serialNumber
    certInputTable["cert.format"] = format
    certInputTable["cert.certType"] = "self"
    certInputTable["cert.certData"] = certData
    valid = db.insert("cert",certInputTable)

    -- return
	if (valid) then
		db.commitTransaction(true)		
		return "OK", "X509_SELF_ADD_SUCCESS"
	else
		db.rollback()
		return "ERROR", "X509_SELF_ADD_FAILURE"
	end
end
